/*
 * Project Kimchi
 *
 * Copyright IBM, Corp. 2013
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
var kimchi = {

    url : "../../../",

    widget: {},

    /**
     * A wrapper of jQuery.ajax function to allow custom bindings.
     *
     * @param settings an extended object to jQuery Ajax settings object
     *   with some extra properties (see below)
     *
     *   resend: if the XHR has failed due to 401, the XHR can be resent
     *     after user being authenticated successfully by setting resend
     *     to true: settings = {resend: true}. It's useful for switching
     *     pages (Guests, Templates, etc.).
     *       e.g., the user wants to list guests by clicking Guests tab,
     *     but he is told not authorized and a login window will pop up.
     *     After login, the Ajax request for /vms will be resent without
     *     user clicking the tab again.
     *       Default to false.
     */
    requestJSON : function(settings) {
        settings['originalError'] = settings['error'];
        settings['error'] = null;
        settings['kimchi'] = true;
        return $.ajax(settings);
    },

    /**
     *
     * Get host capabilities
     * suc: callback if succeed err: callback if failed
     */
    getCapabilities : function(suc, err, done) {
        done = typeof done !== 'undefined' ? done: function(){};
        kimchi.requestJSON({
            url : "/config/capabilities",
            type : "GET",
            contentType : "application/json",
            dataType : "json",
            success: suc,
            error: err,
            complete: done
        });
    },

    /**
     * Get the host static information.
     */
    getHost: function(suc, err) {
        kimchi.requestJSON({
            url : kimchi.url + 'host',
            type : 'GET',
            resend: true,
            contentType : 'application/json',
            dataType : 'json',
            success : suc,
            error: err
        });
    },

    /**
     * Get the dynamic host stats (usually used for monitoring).
     */
    getHostStats : function(suc, err) {
        kimchi.requestJSON({
            url : kimchi.url + 'host/stats',
            type : 'GET',
            contentType : 'application/json',
            headers: {'Kimchi-Robot': 'kimchi-robot'},
            dataType : 'json',
            success : suc,
            error: err
        });
    },

    /**
     *
     * Create a new Virtual Machine. Usage: kimchi.createVM({ name: 'MyUbuntu',
     * template: '/templates/ubuntu_base' }, creationSuc, creationErr);
     *
     * settings: name *(optional)*: The name of the VM. Used to identify the VM
     * in this API. If omitted, a name will be chosen based on the template
     * used. template: The URI of a Template to use when building the VM
     * storagepool *(optional)*: Assign a specific Storage Pool to the new VM
     * suc: callback if succeed err: callback if failed
     */
    createVM : function(settings, suc, err) {
        kimchi.requestJSON({
            url : "/vms",
            type : "POST",
            contentType : "application/json",
            data : JSON.stringify(settings),
            dataType : "json"
        }).done(suc).fail(err);
    },

    /**
     *
     * Create a new Template. settings name: The name of the Template. Used to
     * identify the Template in this API suc: callback if succeed err: callback
     * if failed
     */
    createTemplate : function(settings, suc, err) {
        kimchi.requestJSON({
            url : "/templates",
            type : "POST",
            contentType : "application/json",
            data : JSON.stringify(settings),
            dataType : "json",
            success: suc,
            error: err
        });
    },

    deleteTemplate : function(tem, suc, err) {
        kimchi.requestJSON({
            url : kimchi.url + 'templates/' + encodeURIComponent(tem),
            type : 'DELETE',
            contentType : 'application/json',
            dataType : 'json',
            success : suc,
            error : err
        });
    },

    listTemplates : function(suc, err) {
        kimchi.requestJSON({
            url : kimchi.url + 'templates',
            type : 'GET',
            contentType : 'application/json',
            dataType : 'json',
            success : suc,
            error : err
        });
    },

    /**
     * Retrieve the information of a template by the given name.
     */
    retrieveTemplate : function(templateName, suc, err) {
        kimchi.requestJSON({
            url : kimchi.url + "templates/" + encodeURIComponent(templateName),
            type : 'GET',
            contentType : 'application/json',
            dataType : 'json'
        }).done(suc);
    },

    /**
     * Update a template with new information. TODO: Update me when the RESTful
     * API is available. Now work it around by remove the template and then
     * recreate it with new information.
     */
    updateTemplate : function(name, settings, suc, err) {
        $.ajax({
            url : kimchi.url + "templates/" + encodeURIComponent(name),
            type : 'PUT',
            contentType : 'application/json',
            data : JSON.stringify(settings),
            dataType : 'json'
        }).done(suc).fail(err);
    },

    /**
     * Create a new Storage Pool. settings name: The name of the Storage Pool
     * path: The path of the defined Storage Pool type: The type of the defined
     * Storage Pool capacity: The total space which can be used to store volumes
     * The unit is MBytes suc: callback if succeed err: callback if failed
     */
    createStoragePool : function(settings, suc, err) {
        kimchi.requestJSON({
            url : '/storagepools',
            type : 'POST',
            contentType : 'application/json',
            data : JSON.stringify(settings),
            dataType : 'json'
        }).done(suc).fail(err);
    },

    startVM : function(vm, suc, err) {
        kimchi.requestJSON({
            url : kimchi.url + 'vms/' + encodeURIComponent(vm) + '/start',
            type : 'POST',
            contentType : 'application/json',
            dataType : 'json',
            success : suc,
            error : err
        });
    },

    stopVM : function(vm, suc, err) {
        kimchi.requestJSON({
            url : kimchi.url + 'vms/' + encodeURIComponent(vm) + '/stop',
            type : 'POST',
            contentType : 'application/json',
            dataType : 'json',
            success : suc,
            error : err
        });
    },

    resetVM : function(vm, suc, err) {
        kimchi.requestJSON({
            url : kimchi.url + 'vms/' + encodeURIComponent(vm) + '/stop',
            type : 'POST',
            contentType : 'application/json',
            dataType : 'json',
            success : function() {
                kimchi.requestJSON({
                    url : kimchi.url + 'vms/' + encodeURIComponent(vm) + '/start',
                    type : 'POST',
                    contentType : 'application/json',
                    dataType : 'json',
                    success : suc,
                    error : err
                });
            },
            error : err
        });
    },

    /**
     * Retrieve the information of a given VM by its name.
     *
     * @param vm VM name
     * @param suc callback for success
     * @param err callback for error
     */
    retrieveVM : function(vm, suc, err) {
        $.ajax({
            url : kimchi.url + 'vms/' + encodeURIComponent(vm),
            type : 'GET',
            contentType : 'application/json',
            dataType : 'json',
            success: suc,
            error: err
        });
    },

    /**
     * Update a VM with new information.
     */
    updateVM : function(name, settings, suc, err) {
        $.ajax({
            url : kimchi.url + "vms/" + encodeURIComponent(name),
            type : 'PUT',
            contentType : 'application/json',
            data : JSON.stringify(settings),
            dataType : 'json',
            success: suc,
            error: err
        });
    },

    deleteVM : function(vm, suc, err) {
        kimchi.requestJSON({
            url : kimchi.url + 'vms/' + encodeURIComponent(vm),
            type : 'DELETE',
            contentType : 'application/json',
            dataType : 'json',
            success : suc,
            error : err
        });
    },

    vncToVM : function(vm) {
        kimchi.requestJSON({
            url : '/config',
            type : 'GET',
            dataType : 'json'
        }).done(function(data, textStatus, xhr) {
            proxy_port = data['display_proxy_port'];
            kimchi.requestJSON({
                url : "/vms/" + encodeURIComponent(vm) + "/connect",
                type : "POST",
                dataType : "json"
            }).done(function() {
                url = 'https://' + location.hostname + ':' + proxy_port;
                url += "/console.html?url=vnc_auto.html&port=" + proxy_port;
                url += "&path=?token=" + encodeURIComponent(vm);
                url += "&kimchi=" + location.port;
                url += '&encrypt=1';
                window.open(url);
            });
        }).error(function() {
            kimchi.message.error.code('KCHAPI6002E');
        });
    },

    spiceToVM : function(vm) {
        kimchi.requestJSON({
            url : '/config',
            type : 'GET',
            dataType : 'json'
        }).done(function(data, textStatus, xhr) {
            proxy_port = data['display_proxy_port'];
            kimchi.requestJSON({
                url : "/vms/" + encodeURIComponent(vm) + "/connect",
                type : "POST",
                dataType : "json"
            }).done(function(data, textStatus, xhr) {
                url = 'https://' + location.hostname + ':' + proxy_port;
                url += "/console.html?url=spice.html&port=" + proxy_port;
                url += "&listen=" + location.hostname;
                url += "&token=" + encodeURIComponent(vm);
                url += "&kimchi=" + location.port;
                url += '&encrypt=1';
                window.open(url);
            });
        }).error(function() {
            kimchi.message.error.code('KCHAPI6002E');
        });
    },

    listVMs : function(suc, err) {
        kimchi.requestJSON({
            url : kimchi.url + 'vms',
            type : 'GET',
            contentType : 'application/json',
            headers: {'Kimchi-Robot': 'kimchi-robot'},
            dataType : 'json',
            resend: true,
            success : suc,
            error : err
        });
    },

    listTemplates : function(suc, err) {
        kimchi.requestJSON({
            url : kimchi.url + 'templates',
            type : 'GET',
            contentType : 'application/json',
            dataType : 'json',
            resend: true,
            success : suc,
            error : err
        });
    },

    listStoragePools : function(suc, err) {
        kimchi.requestJSON({
            url : kimchi.url + 'storagepools',
            type : 'GET',
            contentType : 'application/json',
            dataType : 'json',
            resend: true,
            success : suc,
            error : err
        });
    },

    listStorageVolumes : function(poolName, suc, err) {
        $.ajax({
            url : kimchi.url + 'storagepools/' + encodeURIComponent(poolName) + '/storagevolumes',
            type : 'GET',
            contentType : 'application/json',
            dataType : 'json',
            success : suc,
            error : err
        });
    },

    listIsos : function(suc, err) {
        kimchi.requestJSON({
            url : kimchi.url + 'storagepools/kimchi_isos/storagevolumes',
            type : 'GET',
            contentType : 'application/json',
            dataType : 'json',
            success : suc,
            error : err
        });
    },

    listDistros : function(suc, err) {
        kimchi.requestJSON({
            url : kimchi.url + 'config/distros',
            type : 'GET',
            contentType : 'application/json',
            dataType : 'json',
            success : suc,
            error : err
        });
    },

    stepListDeepScanIsos : function(suc, err) {
        var deepScanHandler = {
            stop : false
        };
        var isoPool = 'iso' + new Date().getTime();
        kimchi.createStoragePool({
            name : isoPool,
            type : 'kimchi-iso',
            path : '/'
        }, function(result) {
            var taskId = result.task_id;
            function monitorTask() {
                if (deepScanHandler.stop) {
                    return;
                }
                kimchi.getTask(taskId, function(result) {
                    var status = result.status;
                    if (status === "finished") {
                        if (deepScanHandler.stop) {
                            return;
                        }
                        kimchi.listStorageVolumes(isoPool, function(isos) {
                            if (deepScanHandler.stop) {
                                return;
                            }
                            suc(isos, true);
                        }, err);
                    } else if (status === "running") {
                        if (deepScanHandler.stop) {
                            return;
                        }
                        kimchi.listStorageVolumes(isoPool, function(isos) {
                            if (deepScanHandler.stop) {
                                return;
                            }
                            suc(isos, false);
                            setTimeout(monitorTask, 2000);
                        }, err);
                    } else if (status === "failed") {
                        if (deepScanHandler.stop) {
                            return;
                        }
                        err(result.message);
                    }
                }, err);
            }
            setTimeout(monitorTask, 2000);
        }, err);
        return deepScanHandler;
    },

    listDeepScanIsos : function(suc, err) {
        var isoPool = 'iso' + new Date().getTime();
        kimchi.createStoragePool({
            name : isoPool,
            type : 'kimchi-iso',
            path : '/'
        }, function(result) {
            var taskId = result.task_id;
            function monitorTask() {
                kimchi.getTask(taskId, function(result) {
                    var status = result.status;
                    if (status === "finished") {
                        kimchi.listStorageVolumes(isoPool, suc, err);
                    } else if (status === "running") {
                        setTimeout(monitorTask, 50);
                    } else if (status === "failed") {
                        err(result.message);
                    }
                }, err);
            }
            monitorTask();
        }, err);
    },

    getTask : function(taskId, suc, err) {
        kimchi.requestJSON({
            url : kimchi.url + 'tasks/' + encodeURIComponent(taskId),
            type : 'GET',
            contentType : 'application/json',
            dataType : 'json',
            success : suc,
            error : err
        });
    },

    login : function(settings, suc, err) {
        $.ajax({
            url : "/login",
            type : "POST",
            contentType : "application/json",
            data : JSON.stringify(settings),
            dataType : "json"
        }).done(suc).fail(err);
    },

    logout : function(suc, err) {
        kimchi.requestJSON({
            url : '/logout',
            type : 'POST',
            contentType : "application/json",
            dataType : "json"
        }).done(suc).fail(err);
    },

    deleteStoragePool : function(poolName, suc, err) {
        $.ajax({
            url : kimchi.url + 'storagepools/' + encodeURIComponent(poolName),
            type : 'DELETE',
            contentType : 'application/json',
            dataType : 'json',
            success : suc,
            error : err
        });
    },

    changePoolState : function(poolName, state, suc, err) {
        if (state === 'activate' || state === 'deactivate')
            $.ajax({
                url : kimchi.url + 'storagepools/' + encodeURIComponent(poolName) + '/' + state,
                type : 'POST',
                contentType : 'application/json',
                dataType : 'json',
                success : suc,
                error : err
            });
    },

    listPlugins : function(suc, err) {
        kimchi.requestJSON({
            url : kimchi.url + 'plugins',
            type : 'GET',
            contentType : 'application/json',
            dataType : 'json',
            async: false,
            resend: true,
            success : suc,
            error : err
        });
     },

    listNetworks : function(suc, err) {
        kimchi.requestJSON({
            url : kimchi.url + 'networks',
            type : 'GET',
            contentType : 'application/json',
            dataType : 'json',
            resend : true,
            success : suc,
            error : err ? err : function(data) {
                kimchi.message.error(data.responseJSON.reason);
            }
        });
    },

    toggleNetwork : function(name, on, suc, err) {
        var action = on ? "activate" : "deactivate";
        kimchi.requestJSON({
            url : kimchi.url + 'networks/' + encodeURIComponent(name) + '/' + action,
            type : 'POST',
            contentType : 'application/json',
            dataType : 'json',
            success : suc,
            error : err ? err : function(data) {
                kimchi.message.error(data.responseJSON.reason);
            }
        });
    },

    createNetwork : function(network, suc, err) {
        kimchi.requestJSON({
            url : kimchi.url + 'networks',
            type : 'POST',
            contentType : 'application/json',
            dataType : 'json',
            data : JSON.stringify(network),
            success : suc,
            error : err ? err : function(data) {
                kimchi.message.error(data.responseJSON.reason);
            }
        });
    },

    getInterfaces : function(suc, err) {
        kimchi.requestJSON({
            url : kimchi.url + 'interfaces',
            type : 'GET',
            contentType : 'application/json',
            dataType : 'json',
            resend : true,
            success : suc,
            error : err ? err : function(data) {
                kimchi.message.error(data.responseJSON.reason);
            }
        });
    },

    deleteNetwork : function(name, suc, err) {
        kimchi.requestJSON({
            url : kimchi.url + 'networks/' + encodeURIComponent(name),
            type : 'DELETE',
            contentType : 'application/json',
            dataType : 'json',
            success : suc,
            error : err ? err : function(data) {
                kimchi.message.error(data.responseJSON.reason);
            }
        });
    },

    listReports : function(suc, err) {
        kimchi.requestJSON({
            url : kimchi.url + 'debugreports',
            type : 'GET',
            contentType : 'application/json',
            dataType : 'json',
            resend: true,
            success : suc,
            error : err
        });
    },

    createReport: function(settings, suc, err) {
        var taskID = -1;
        var onTaskResponse = function(result) {
            var taskStatus = result['status'];
            switch(taskStatus) {
            case 'running':
                if(kimchi.stopTrackingReport === true) {
                    return;
                }
                setTimeout(function() {
                    trackTask();
                }, 200);
                break;
            case 'finished':
                suc(result);
                break;
            case 'failed':
                err(result);
                break;
            default:
                break;
            }
        };

        var trackTask = function() {
            kimchi.getTask(taskID, onTaskResponse, err);
        };

        var onResponse = function(data) {
            taskID = data['id'];
            trackTask();
        };

        kimchi.requestJSON({
            url : kimchi.url + 'debugreports',
            type : "POST",
            contentType : "application/json",
            data : JSON.stringify(settings),
            dataType : "json",
            success : onResponse,
            error : err
        });
    },

    deleteReport: function(settings, suc, err) {
        var reportName = encodeURIComponent(settings['name']);
        kimchi.requestJSON({
            url : kimchi.url + 'debugreports/' + reportName,
            type : 'DELETE',
            contentType : 'application/json',
            dataType : 'json',
            success : suc,
            error : err
        });
    },

    downloadReport: function(settings, suc, err) {
        window.open(settings['file']);
    },

    shutdown: function(settings, suc, err) {
        var reboot = settings && settings['reboot'] === true;
        var url = kimchi.url + 'host/' + (reboot ? 'reboot' : 'shutdown');
        kimchi.requestJSON({
            url : url,
            type : 'POST',
            contentType : 'application/json',
            dataType : 'json',
            success : suc,
            error : err
        });
    },

    listHostPartitions : function(suc, err) {
        kimchi.requestJSON({
            url : kimchi.url + 'host/partitions',
            type : 'GET',
            contentType : 'application/json',
            dataType : 'json',
            success : suc,
            error : err
        });
    },

    getStorageServers: function(type, suc, err) {
        var url = kimchi.url + 'storageservers?_target_type=' + type;
        kimchi.requestJSON({
            url : url,
            type : 'GET',
            contentType : 'application/json',
            dataType : 'json',
            success : suc,
            error : err
        });
    },

    getStorageTargets: function(server,type, suc, err) {
        var url = kimchi.url + 'storageservers/' + server + '/storagetargets?_target_type=' + type;
        kimchi.requestJSON({
            url : url,
            type : 'GET',
            contentType : 'application/json',
            timeout: 2000,
            dataType : 'json',
            success : suc,
            error : err
        });
    },

    addVMStorage : function(settings, suc, err) {
        var vm = encodeURIComponent(settings['vm']);
        delete settings['vm'];
        kimchi.requestJSON({
            url : kimchi.url + 'vms/' + vm + '/storages',
            type : 'POST',
            contentType : 'application/json',
            data : JSON.stringify(settings),
            dataType : 'json',
            success : suc,
            error : err
        });
    },

    retrieveVMStorage : function(settings, suc, err) {
        var vm = encodeURIComponent(settings['vm']);
        var dev = encodeURIComponent(settings['dev']);
        kimchi.requestJSON({
            url : kimchi.url + "vms/" + vm + '/storages/' + dev,
            type : 'GET',
            contentType : 'application/json',
            dataType : 'json',
            success: suc,
            error: err
        });
    },

    replaceVMStorage : function(settings, suc, err) {
        var vm = encodeURIComponent(settings['vm']);
        var dev = encodeURIComponent(settings['dev']);
        kimchi.requestJSON({
            url : kimchi.url + 'vms/' + vm + '/storages/' + dev,
            type : 'PUT',
            contentType : 'application/json',
            data : JSON.stringify({
                path: settings['path']
            }),
            dataType : 'json',
            success : suc,
            error : err
        });
    },

    deleteVMStorage : function(settings, suc, err) {
        var vm = settings['vm'];
        var dev = settings['dev'];
        kimchi.requestJSON({
            url : kimchi.url + 'vms/' + encodeURIComponent(vm) +
                      '/storages/' + encodeURIComponent(dev),
            type : 'DELETE',
            contentType : 'application/json',
            dataType : 'json',
            success : suc,
            error : err
        });
    },

    listVMStorages : function(params, suc, err) {
        var vm = encodeURIComponent(params['vm']);
        var type = params['storageType'];
        var url = kimchi.url + 'vms/' + vm + '/storages';
        if(type) {
            url += '?type=' + type;
        }
        kimchi.requestJSON({
            url : url,
            type : 'GET',
            contentType : 'application/json',
            dataType : 'json',
            success : suc,
            error : err
        });
    },

    listSoftwareUpdates : function(suc, err) {
        kimchi.requestJSON({
            url : kimchi.url + 'host/packagesupdate',
            type : 'GET',
            contentType : 'application/json',
            dataType : 'json',
            resend: true,
            success : suc,
            error : err
        });
    },

    updateSoftware : function(suc, err, progress) {
        var taskID = -1;
        var onResponse = function(data) {
            taskID = data['id'];
            trackTask();
        };

        var trackTask = function() {
            kimchi.getTask(taskID, onTaskResponse, err);
        };

        var onTaskResponse = function(result) {
            var taskStatus = result['status'];
            switch(taskStatus) {
            case 'running':
                progress && progress(result);
                setTimeout(function() {
                    trackTask();
                }, 200);
                break;
            case 'finished':
            case 'failed':
                suc(result);
                break;
            default:
                break;
            }
        };

        kimchi.requestJSON({
            url : kimchi.url + 'host/swupdate',
            type : "POST",
            contentType : "application/json",
            dataType : "json",
            success : onResponse,
            error : err
        });
    },

    createRepository : function(settings, suc, err) {
        kimchi.requestJSON({
            url : "host/repositories",
            type : "POST",
            contentType : "application/json",
            data : JSON.stringify(settings),
            dataType : "json",
            success: suc,
            error: err
        });
    },

    retrieveRepository : function(repository, suc, err) {
        var reposID = encodeURIComponent(repository);
        kimchi.requestJSON({
            url : kimchi.url + "host/repositories/" + reposID,
            type : 'GET',
            contentType : 'application/json',
            dataType : 'json',
            success : suc,
            error : err
        });
    },

    updateRepository : function(name, settings, suc, err) {
        var reposID = encodeURIComponent(name);
        $.ajax({
            url : kimchi.url + "host/repositories/" + reposID,
            type : 'PUT',
            contentType : 'application/json',
            data : JSON.stringify(settings),
            dataType : 'json',
            success : suc,
            error : err
        });
    },

    enableRepository : function(name, enable, suc, err) {
        var reposID = encodeURIComponent(name);
        $.ajax({
            url : kimchi.url + "host/repositories/" + reposID +
                '/' + (enable === true ? 'enable' : 'disable'),
            type : 'POST',
            contentType : 'application/json',
            dataType : 'json',
            success : suc,
            error : err
        });
    },

    deleteRepository : function(repository, suc, err) {
        var reposID = encodeURIComponent(repository);
        kimchi.requestJSON({
            url : kimchi.url + 'host/repositories/' + reposID,
            type : 'DELETE',
            contentType : 'application/json',
            dataType : 'json',
            success : suc,
            error : err
        });
    },

    listRepositories : function(suc, err) {
        kimchi.requestJSON({
            url : kimchi.url + 'host/repositories',
            type : 'GET',
            contentType : 'application/json',
            dataType : 'json',
            resend: true,
            success : suc,
            error : err
        });
    },

    getHostPCIDevices: function(suc, err) {
        var url = kimchi.url+'host/devices?_cap=fc_host';
        kimchi.requestJSON({
            url : url,
            type : 'GET',
            contentType : 'application/json',
            dataType : 'json',
            success : suc,
            error : err ? err : function(data) {
                kimchi.message.error(data.responseJSON.reason);
            }
        });
    }
};
/*
 * Project Kimchi
 *
 * Copyright IBM, Corp. 2013
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
kimchi.cookie = {
    set: function(key, value, expireDays) {
        value = encodeURIComponent(value);
        value += '; secure'
        if (expireDays) {
            var expireDate = new Date();
            expireDate.setDate(expireDate.getDate() + expireDays);
            value += '; expires=' + expireDate.toUTCString();
        }
        document.cookie = key + '=' + value;
    },

    get: function(key) {
        var cookieRe = new RegExp(';?\\\s*(' + key + ')=(\s*[^;]*);?', 'g');
        var match = cookieRe.exec(document.cookie);
        return match ? decodeURIComponent(match[2]) : undefined;
    },

    remove: function(key) {
        var utcString = new Date().toUTCString();
        document.cookie = key + '=; expires=' + utcString;
    }
};
/*
 * Project Kimchi
 *
 * Copyright IBM, Corp. 2013
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
(function($) {
    $.fn.serializeObject = function() {
        var formDataArray = $(this).serializeArray();
        var formData = {};
        $.each(formDataArray, function(index, data) {
            var names=kimchi.form.parseFormName(data.name);
            kimchi.form.assignValue(names,data.value,formData);
        });
        return formData;
    };
}(jQuery));

kimchi.form = {};
kimchi.form.assignValue = function(names, value, obj) {
    var result=value;

    if(names.length!=0) {
        result=obj;
        var name=names.shift();
        if(!result) {
            result={};
        }
        if(!result[name]) {
            result[name]=kimchi.form.assignValue(names,value);
        }
        else if(names.length==0) {
            if(Array.isArray(result[name])){
                result[name].push(value);
            }
            else {
                result[name]=[result[name],value];
            }
        }
        else {
            result[name]=kimchi.form.assignValue(names,value,result[name]);
        }
    }
    return(result);
}

kimchi.form.parseFormName = function(name, parsedName) {
    if (!parsedName) {
        parsedName=[];
    }
    if(!name || name=="") {
        return(parsedName);
    }
    var openBracket=name.indexOf("[");
    if (openBracket!=-1) {
        var id=name.slice(0, openBracket);
        parsedName.push(id);
        var closeBracket=name.lastIndexOf("]");
        if (closeBracket==-1) {
            closeBracket=name.length;
        }
        var tmpName=name.slice(openBracket+1,closeBracket);
        kimchi.form.parseFormName(tmpName,parsedName);
    }
    else {
        parsedName.push(name);
    }
    return(parsedName);
}
/*
 * Project Kimchi
 *
 * Copyright IBM, Corp. 2013
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
kimchi.widget.Grid = function(params) {
    var containerID = params['container'];
    var container = $('#' + containerID);
    var gridID = params['id'];
    var rowSelection = params['rowSelection'] || 'single';
    var rowListener = params['onRowSelected'];
    var htmlStr = [
      '<div id="', gridID, '" class="grid">',
        '<div class="grid-content">',
          '<div class="grid-header">',
            '<div class="grid-frozen-header-view">',
              '<table class="grid-frozen-header-container">',
              '</table>',
            '</div>',
            '<div class="grid-header-view">',
              '<div class="grid-header-wrapper">',
                '<table class="grid-header-container">',
                '</table>',
              '</div>',
            '</div>',
          '</div>',
          '<div class="grid-body">',
            '<div class="grid-frozen-body-view">',
              '<div class="grid-frozen-body-wrapper">',
                '<table class="grid-frozen-body-container">',
                '</table>',
              '</div>',
            '</div>',
            '<div class="grid-body-view">',
              '<div class="grid-body-wrapper">',
                '<table class="grid-body-container">',
                '</table>',
              '</div>',
            '</div>',
          '</div>',
          '<div class="grid-resizer-leftmost hidden"></div>',
          '<div class="grid-resizer hidden"></div>',
        '</div>',
        '<div class="grid-footer"></div>',
        '<div class="grid-mask hidden">',
            '<div class="grid-loading">',
                '<div class="grid-loading-icon"></div>',
                '<div class="grid-loading-text">',
                    i18n['KCHGRD6001M'],
                '</div>',
            '</div>',
        '</div>',
        '<div class="grid-message hidden">',
          '<div class="grid-message-text">',
            i18n['KCHGRD6002M'],
            '<button class="retry-button btn-small">',
              i18n['KCHGRD6003M'],
            '</button>',
          '</div>',
          '<div class="detailed-title">',
            i18n['KCHGRD6004M'],
          '</div>',
          '<div class="detailed-text"></div>',
        '</div>',
      '</div>'
    ];

    var gridNode = $(htmlStr.join(''))
        .appendTo(container);

    var height = gridNode.height();
    var width = gridNode.width();

    var title = params['title'];
    var titleNode = null;
    if(title) {
        titleNode = $('<div class="grid-caption">' + title + '</div>')
            .prependTo(gridNode);
    }

    var toolbarButtons = params['toolbarButtons'];
    var toolbarNode = null;
    if(toolbarButtons) {
        toolbarNode = $('<div class="grid-toolbar"></div>');
        if(titleNode) {
            titleNode.after(toolbarNode);
        }
        else {
            toolbarNode.prependTo(gridNode);
        }

        $.each(toolbarButtons, function(i, button) {
            var btnHTML = [
                '<button',
                button['id'] ? (' id="' + button['id'] + '"') : '',
                ' class="grid-toolbar-button',
                    button['class'] ? (' ' + button['class']) : '',
                    '"',
                    button['disabled'] === true ? ' disabled' : '',
                    '>',
                    button['label'],
                '</button>'
            ].join('');
            var btnNode = $(btnHTML).appendTo(toolbarNode);
            button['onClick'] &&
                btnNode.on('click', button['onClick']);
        });
    }

    var setupHeaders = function(header, body, fields) {
        var colGroup = $('<colgroup></colgroup>').appendTo(header);
        var headerHeader = $('<thead></thead>');
        var headerRow = $('<tr></tr>').appendTo(headerHeader);
        $.each(fields || [], function(i, field) {
            $('<col class="' +
                field['class'] +
            '"/>')
                .appendTo(colGroup);
            $('<th><div class="cell-text-wrapper">' +
                field['label'] +
            '</div></th>').appendTo(headerRow);
        });
        headerHeader.appendTo(header);

        var totalWidth = 0;
        $('col', header).each(function(index, col) {
            var width = $(col).width();
            totalWidth += width;
            $(col).css('width', width + 'px');
        });
        $('colgroup', header).clone().appendTo(body);
        return totalWidth;
    };

    var frozenHeaderContainer = $('.grid-frozen-header-container', gridNode);
    var frozenBodyContainer = $('.grid-frozen-body-container', gridNode);
    var frozenWidth = setupHeaders(
        frozenHeaderContainer,
        frozenBodyContainer,
        params['frozenFields']
    );

    var headerContainer = $('.grid-header-container', gridNode);
    var bodyContainer = $('.grid-body-container', gridNode);
    setupHeaders(headerContainer, bodyContainer, params['fields']);

    $.each([
        frozenHeaderContainer,
        headerContainer,
        frozenBodyContainer,
        bodyContainer
    ], function(i, tableNode) {
        $(tableNode).css('table-layout', 'auto');
    });

    var gridContentNode = $('.grid-content', gridNode);
    var captionHeight = titleNode && $(titleNode).height() || 0;
    var toolbarHeight = toolbarNode && $(toolbarNode).height() || 0;
    gridContentNode.css('top', (captionHeight + toolbarHeight) + 'px');

    var maskNode = $('.grid-mask', gridNode);
    maskNode.css('top', captionHeight + 'px');

    var messageNode = $('.grid-message', gridNode);
    messageNode.css('top', captionHeight + 'px');


    var getValue = function(name, obj) {
    var result=undefined;
    if(!Array.isArray(name)) {
        name=kimchi.form.parseFormName(name);
    }
    if(name.length!=0) {
        var tmpName=name.shift();
        if(obj[tmpName]!=undefined) {
            result=obj[tmpName];
        }
        if(name.length!=0) {
            result=getValue(name,obj[tmpName]);
        }
    }
    return(result);
    };

    var fillBody = function(container, fields, data) {
        var tbody = ($('tbody', container).length && $('tbody', container))
            || $('<tbody></tbody>').appendTo(container);
        tbody.empty();
        $.each(data, function(i, row) {
            var rowNode = $('<tr></tr>').appendTo(tbody);
            $.each(fields, function(fi, field) {
                var value = getValue(field['name'], row);
                $('<td><div class="cell-text-wrapper"' +
                     (field['makeTitle'] === true
                         ? ' title="' + value + '"'
                         : ''
                     ) + '>' + value.toString() + '</div></td>'
                ).appendTo(rowNode);
            });
        });
    };

    var frozenHeaderView = $('.grid-frozen-header-view', gridNode);
    var headerView = $('.grid-header-view', gridNode);
    var bodyView = $('.grid-body-view', gridNode);
    headerView.css('left', (frozenWidth) + 'px');
    bodyView.css('left', (frozenWidth) + 'px');

    var bodyWidth = width - frozenWidth;
    headerContainer.css('width', bodyWidth + 'px');
    bodyContainer.css('width', bodyWidth + 'px');

    var fixTableLayout = function() {
        $.each([
            frozenHeaderContainer,
            headerContainer,
            frozenBodyContainer,
            bodyContainer
        ], function(i, tableNode) {
            $(tableNode).css('table-layout', 'fixed');
        });
    };
    fixTableLayout();

    var gridHeader = $('.grid-header', gridNode);
    $('th', gridHeader).on('mouseover mousemove', function(event) {
        var pageX = event.pageX;
        var tailPos = $(this).width() + $(this).offset()['left'];
        var atResizer = Math.abs(pageX - tailPos) <= 2;
        var isResizing = !resizer.hasClass('hidden');
        $('body')[(atResizer || isResizing)
            ? 'addClass'
            : 'removeClass'
        ]('resizing');
    });

    $('th', gridHeader).on('mouseout', function(event) {
        resizer.hasClass('hidden') &&
            $('body').removeClass('resizing');
    });

    var gridBody = $('.grid-body', gridNode);
    var contentHeight = gridContentNode.height();
    var resizerLeftmost = $('.grid-resizer-leftmost', gridNode);
    var resizer = $('.grid-resizer', gridNode);
    resizerLeftmost.css('height', contentHeight + 'px');
    resizer.css('height', contentHeight + 'px');

    var stylingRow = function(row, className, add) {
        var index = $(row).index() + 1;
        $('tr', frozenBodyContainer)
            .removeClass(className);
        $('tr', bodyContainer)
            .removeClass(className);

        if(add === false) {
            return;
        }

        $('tr:nth-child(' + index + ')', frozenBodyContainer)
            .addClass(className);
        $('tr:nth-child(' + index + ')', bodyContainer)
            .addClass(className);
    };

    var selectedIndex = -1;
    var setBodyListeners = function() {
        if(rowSelection != 'disabled') {
            $('tr', gridBody).on('mouseover', function(event) {
                stylingRow(this, 'hover');
            });

            $('tr', gridBody).on('mouseout', function(event) {
                stylingRow(this, 'hover', false);
            });

            $('tr', gridBody).on('click', function(event) {
                selectedIndex = $(this).index();
                stylingRow(this, 'selected');
                rowListener && rowListener();
            });
        }

        $('.grid-body-view', gridNode).on('scroll', function(event) {
            $('.grid-header .grid-header-view', gridNode)
                .prop('scrollLeft', this.scrollLeft);
            $('.grid-body .grid-frozen-body-view', gridNode)
                .prop('scrollTop', this.scrollTop);
        });
    };

    this.frozenFields = params['frozenFields'];
    this.fields = params['fields'];
    this.setData = function(data) {
        this.data = data;
        fillBody(frozenBodyContainer, this.frozenFields, data);
        fillBody(bodyContainer, this.fields, data);
        setBodyListeners();
    };

    this.getSelected = function() {
        return selectedIndex >= 0
            ? this.data[selectedIndex]
            : null;
    };

    var columnBeingResized = null;
    var CONTAINER_NORMAL = 0, CONTAINER_FROZEN = 1;
    var containerBeingResized = CONTAINER_NORMAL;
    var startResizing = function(container, event) {
        if(!($('body').hasClass('resizing') && resizer.hasClass('hidden'))) {
            return;
        }

        columnBeingResized = container;
        var pageX = event.pageX;
        var gridOffsetX = gridNode.offset()['left'];
        var leftmostOffsetX = $(container).offset()['left'] - gridOffsetX;
        var left = pageX - gridOffsetX;
        resizerLeftmost.css('left', leftmostOffsetX + 'px');
        resizer.css('left', left + 'px');
        resizerLeftmost.removeClass('hidden');
        resizer.removeClass('hidden');
        event.preventDefault();
    };

    var endResizing = function(event) {
        if(!$('body').hasClass('resizing')) {
            return;
        }
        resizerLeftmost.addClass('hidden');
        resizer.addClass('hidden');
        $('body').removeClass('resizing');
        var leftmostOffset = $(columnBeingResized).offset()['left'];
        var left = event.pageX;
        if(leftmostOffset > left) {
            return;
        }

        resizeColumnWidth(
            $(columnBeingResized).index(),
            left - leftmostOffset
        );
        fixTableLayout();
        columnBeingResized = null;
    };

    var resizeColumnWidth = function(index, width) {
        var width = Math.ceil(width);
        var widthArray = [];
        var totalWidth = 0;
        var header = headerContainer;
        var body = bodyContainer;
        if(containerBeingResized === CONTAINER_FROZEN) {
            header = frozenHeaderContainer;
            body = frozenBodyContainer;
        }
        $('col', header).each(function(i, colNode) {
            var w = index === i ? width : $(colNode).width();
            widthArray.push(w);
            totalWidth += w;
        });
        $.each([header, body], function(i, container) {
            container.css({
                'table-layout': 'fixed',
                width: totalWidth + 'px'
            });
            $('col:nth-child(' + (index + 1) + ')', container).css({
                width: width + 'px'
            });
        });

        if(containerBeingResized === CONTAINER_FROZEN) {
            $.each([headerView, bodyView], function(i, view) {
                view.css({
                    left: totalWidth + 'px'
                });
            });
        }
    };

    $('th', frozenHeaderContainer).on('mousedown', function(event) {
        containerBeingResized = CONTAINER_FROZEN;
        startResizing(this, event);
    });
    $('th', headerContainer).on('mousedown', function(event) {
        containerBeingResized = CONTAINER_NORMAL;
        startResizing(this, event);
    });

    var positionResizer = function(event) {
        if(resizer.hasClass('hidden')) {
            return;
        }

        var pageX = event.pageX;
        var gridOffsetX = gridNode.offset()['left'];
        var leftMost = resizerLeftmost.position()['left'];
        var offsetX = pageX - gridOffsetX;
        offsetX = offsetX >= leftMost ? offsetX : leftMost;
        resizer.css('left', offsetX + 'px');
    };

    $('body').on('mousemove', positionResizer);
    $('body').on('mouseup', endResizing);

    this.showMessage = function(msg) {
        $('.detailed-text', messageNode).text(msg);
        $(messageNode).removeClass('hidden');
    };

    this.hideMessage = function() {
        $(messageNode).addClass('hidden');
    };

    this.destroy = function() {
        $('body').off('mousemove', positionResizer);
        $('body').off('mouseup', endResizing);
    };

    var data = params['data'];
    var self = this;
    var reload = function() {
        if(!data) {
            return;
        }

        $(messageNode).addClass('hidden');

        if($.isArray(data)) {
            self.setData(data);
            return;
        }

        if($.isFunction(data)) {
            var loadData = data;
            maskNode.removeClass('hidden');
            loadData(function(data) {
                self.setData(data);
                maskNode.addClass('hidden');
            });
        }
    };

    var reloadButton = $('.retry-button', gridNode);
    $(reloadButton).on('click', function(event) {
        reload();
    });

    this.reload = reload;
    reload();
};
/*
 * Project Kimchi
 *
 * Copyright IBM, Corp. 2013
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
kimchi.guest_add_main = function() {
    var showTemplates = function() {
        kimchi.topic('templateCreated').unsubscribe(showTemplates);
        kimchi.listTemplates(function(result) {
            if (result && result.length) {
                $('#prompt-create-template').addClass('hidden');
                $('#prompt-choose-template').removeClass('hidden');
                var html = '';
                var tmpl = $('#tmpl-template').html();
                $.each(result, function(index, value) {
                    html += kimchi.template(tmpl, value);
                });
                $('#templateTile').html(html);
                return;
            }

            $('#btn-create-template').on('click', function(event) {
                kimchi.topic('templateCreated').subscribe(showTemplates);

                kimchi.window.open('template-add.html');

                event.preventDefault();
            });

            $('#prompt-choose-template').addClass('hidden');
            $('#prompt-create-template').removeClass('hidden');
        }, function(err) {
            kimchi.message.error(err.responseJSON.reason);
        });
    };

    function validateForm() {
        if (!$('input[name=template]:checked', '#templateTile').val()) {
            return false;
        }
        return true;
    }

    $('#form-vm-add').change(function() {
        if (validateForm()) {
            $('#vm-doAdd').removeAttr('disabled');
        }
    });

    var addGuest = function(event) {
        var formData = $('#form-vm-add').serializeObject();

        kimchi.createVM(formData, function() {
            kimchi.listVmsAuto();
            kimchi.window.close();
        }, function(jqXHR, textStatus, errorThrown) {
            var reason = jqXHR &&
                jqXHR['responseJSON'] &&
                jqXHR['responseJSON']['reason'];
            kimchi.message.error(reason);
        });

        return false;
    };

    $('#form-vm-add').on('submit', addGuest);
    $('#vm-doAdd').on('click', addGuest);

    showTemplates();
};
/*
 * Project Kimchi
 *
 * Copyright IBM, Corp. 2013
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
kimchi.guest_cdrom_edit_main = function() {
    var cdromEditForm = $('#form-vm-cdrom-edit');
    var submitButton = $('#vm-cdrom-button-edit');
    var nameTextbox = $('input[name="dev"]', cdromEditForm);
    var typeTextbox = $('input[name="type"]', cdromEditForm);
    var pathTextbox = $('input[name="path"]', cdromEditForm);
    var originalPath = null;

    kimchi.retrieveVMStorage({
        vm: kimchi.selectedGuest,
        dev: kimchi.selectedGuestStorage
    }, function(storage) {
        for(var prop in storage) {
            $('input[name="' + prop + '"]', cdromEditForm).val(storage[prop]);
        }

        originalPath = storage['path'];
    });

    var submitForm = function(event) {
        if(submitButton.prop('disabled')) {
            return false;
        }

        var path = pathTextbox.val();
        if(!path || path === '') {
            return false;
        }

        $.each([submitButton, nameTextbox, pathTextbox], function(i, c) {
            $(c).prop('disabled', true);
        });
        $(submitButton).addClass('loading').text(i18n['KCHVMCD6005M']);

        var settings = {
            vm: kimchi.selectedGuest,
            dev: kimchi.selectedGuestStorage,
            path: path
        };

        kimchi.replaceVMStorage(settings, function(result) {
            kimchi.window.close();
            kimchi.topic('kimchi/vmCDROMReplaced').publish({
                result: result
            });
        }, function(result) {
            var errText = result['reason'] ||
                result['responseJSON']['reason'];
            kimchi.message.error(errText);

            $.each([submitButton, nameTextbox, pathTextbox], function(i, c) {
                $(c).prop('disabled', false);
            });
            $(submitButton).removeClass('loading').text(i18n['KCHVMCD6004M']);
        });

        event.preventDefault();
    };

    cdromEditForm.on('submit', submitForm);
    submitButton.on('click', submitForm);
    pathTextbox.on('input propertychange', function(event) {
        var invalid =
            ($(this).val() === originalPath) ||
            ($(this).val() === '');
        $(submitButton).prop('disabled', invalid);
    });
};
/*
 * Project Kimchi
 *
 * Copyright IBM, Corp. 2013
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
kimchi.guest_edit_main = function() {
    var buttonContainer = $('#action-button-container');
    $('#guest-edit-tabs').tabs({
        beforeActivate: function(event, ui) {
            var deactivated = ui['oldPanel'];
            if($(deactivated).attr('id') === 'form-guest-edit-general') {
                $(buttonContainer).addClass('hidden');
            }
            else {
                $(buttonContainer).removeClass('hidden');
            }
        }
    });

    var guestEditForm = $('#form-guest-edit-general');
    var saveButton = $('#guest-edit-button-save');

    var refreshCDROMs = function() {
        kimchi.listVMStorages({
            vm: kimchi.selectedGuest,
            storageType: 'cdrom'
        }, function(storages) {
            var rowHTML = $('#cdrom-row-tmpl').html();
            var container = $('#guest-edit-cdrom-row-container');
            $(container).empty();

            $.each(storages, function(index, storage) {
                storage['vm'] = kimchi.selectedGuest;
                var templated = kimchi.template(rowHTML, storage);
                container.append(templated);
            });

            var replaceCDROM = function(event) {
                event.preventDefault();
                kimchi.selectedGuestStorage = $(this).data('dev');
                kimchi.window.open("guest-cdrom-edit.html");
            };

            $('input[type="text"][name="cdrom"]', container).on('click', replaceCDROM);
            $('.replace', container).on('click', replaceCDROM);

            $('.detach', container).on('click', function(e) {
                e.preventDefault();
                var settings = {
                    title : i18n['KCHAPI6004M'],
                    content : i18n['KCHVMCD6001M'],
                    confirm : i18n['KCHAPI6002M'],
                    cancel : i18n['KCHAPI6003M']
                };

                var dev = $(this).data('dev');
                kimchi.confirm(settings, function() {
                    kimchi.deleteVMStorage({
                        vm: kimchi.selectedGuest,
                        dev: dev
                    }, function() {
                        kimchi.topic('kimchi/vmCDROMDetached').publish();
                    });
                });
            });
        });
    };

    var initContent = function(guest) {
        guest['icon'] = guest['icon'] || 'images/icon-vm.png';
        for ( var prop in guest) {
            $('input[name="' + prop + '"]', guestEditForm).val(guest[prop]);
        }

        refreshCDROMs();

        $('#guest-edit-attach-cdrom-button').on('click', function(event) {
            event.preventDefault();
            kimchi.window.open("guest-storage-add.html");
        });

        var onAttached = function(params) {
            refreshCDROMs();
        };
        var onReplaced = function(params) {
            refreshCDROMs();
        };
        var onDetached = function(params) {
            refreshCDROMs();
        };

        kimchi.topic('kimchi/vmCDROMAttached').subscribe(onAttached);
        kimchi.topic('kimchi/vmCDROMReplaced').subscribe(onReplaced);
        kimchi.topic('kimchi/vmCDROMDetached').subscribe(onDetached);

        kimchi.clearGuestEdit = function() {
            kimchi.topic('kimchi/vmCDROMAttached').unsubscribe(onAttached);
            kimchi.topic('kimchi/vmCDROMReplaced').unsubscribe(onReplaced);
            kimchi.topic('kimchi/vmCDROMDetached').unsubscribe(onDetached);
        };
    };

    kimchi.retrieveVM(kimchi.selectedGuest, initContent);

    var submitForm = function(event) {
        $(saveButton).prop('disabled', true);
        var editableFields = [ 'name' ];
        var data = {};
        $.each(editableFields, function(i, field) {
            data[field] = $('input[name="' + field + '"]', guestEditForm).val();
        });
        kimchi.updateVM(kimchi.selectedGuest, data, function() {
            kimchi.listVmsAuto();
            kimchi.window.close();
        }, function(err) {
            kimchi.message.error(err.responseJSON.reason);
        });

        event.preventDefault();
    };

    $(guestEditForm).on('submit', submitForm);
    $(saveButton).on('click', submitForm);
};
/*
 * Project Kimchi
 *
 * Copyright IBM, Corp. 2013
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

kimchi.vmstart = function(event) {
    var button=$(this);
    if (!button.hasClass('loading')) {
        button.addClass('loading');
        var vm=$(this).closest('li[name=guest]');
        var vm_id=vm.attr("id");
        kimchi.startVM(vm_id, function(result) {
            button.removeClass('loading');
            kimchi.listVmsAuto();
            }, function(err) {
                button.removeClass('loading');
                kimchi.message.error(err.responseJSON.reason);
            }
        );
    } else {
        event.preventDefault();
        event.stopPropagation();
        return;
    }
};

kimchi.vmstop = function(event) {
    var button=$(this);
    if (!button.hasClass('loading')) {
        button.addClass('loading');
        var vm=button.closest('li[name=guest]');
        var vm_id=vm.attr("id");
        kimchi.stopVM(vm_id, function(result) {
            button.removeClass('loading');
            kimchi.listVmsAuto();
        }, function(err) {
            kimchi.message.error(err.responseJSON.reason);
        });
    } else {
        event.preventDefault();
        event.stopPropagation();
    }
};

kimchi.vmreset = function(event){
    var vm=$(this).closest('li[name=guest]');
    var vm_id=vm.attr("id");
    kimchi.resetVM(vm_id, function(result) {
            kimchi.listVmsAuto();
        }, function(err) {
            kimchi.message.error(err.responseJSON.reason);
        }
    );
};

kimchi.vmdelete = function(event) {
    var vm = $(this).closest('li[name=guest]');
    var vm_id=vm.attr("id");
    var settings = {
        title : i18n['KCHVM6002M'],
        content : i18n['KCHVM6001M'],
        confirm : i18n['KCHAPI6002M'],
        cancel : i18n['KCHAPI6003M']
    };
    kimchi.confirm(settings, function() {
        kimchi.deleteVM(vm_id, function(result) {
            kimchi.listVmsAuto();
        }, function(err) {
            kimchi.message.error(err.responseJSON.reason);
        });
    }, function() {
    });
};

kimchi.vmmedia = function(event) {
    var vm = $(this).closest('li[name=guest]');
    var vm_id = vm.attr('id');
    kimchi.selectedGuest = vm_id;
    kimchi.window.open({
        url: 'guest-media.html',
        close: function() {
            kimchi.clearGuestMedia();
        }
    });
};

kimchi.vmedit = function(event) {
    var vm = $(this).closest('li[name=guest]');
    var vm_id=vm.attr("id");
    kimchi.selectedGuest = vm_id;
    kimchi.window.open({
        url: 'guest-edit.html',
        close: function() {
            kimchi.clearGuestEdit();
        }
    });
};

kimchi.openVmConsole = function(event) {
    var vm=$(this).closest('li[name=guest]');
    var vmObject=vm.data();
    if (vmObject.graphics['type'] == 'vnc') {
        kimchi.vncToVM(vm.attr('id'));
    }
    else if (vmObject.graphics['type'] == 'spice') {
        kimchi.spiceToVM(vm.attr('id'));
    }

};

kimchi.getVmsCurrentConsoleImgs = function() {
    var res = new Object();
    $('#guestList').children().each(function() {
        res[$(this).attr('id')] = $(this).find('img.imgactive').attr('src');
    })
    return res;
};

kimchi.getOpenMenuVmId = function() {
    var result;
    var openMenu = $('#guestList .open:first')
    if(openMenu) {
        var li_element=openMenu.closest('li');
        result=li_element.attr('id');
    }
    return result;
};

kimchi.listVmsAuto = function() {
    if (kimchi.vmTimeout) {
        clearTimeout(kimchi.vmTimeout);
    }
    kimchi.listVMs(function(result, textStatus, jqXHR) {
        if (result && textStatus=="success") {
            if(result.length) {
                var listHtml = '';
                var guestTemplate = kimchi.guestTemplate;
                var currentConsoleImages = kimchi.getVmsCurrentConsoleImgs();
                var openMenuGuest = kimchi.getOpenMenuVmId();
                $('#guestList').empty();
                $('#guestListField').show();
                $('#noGuests').hide();

                $.each(result, function(index, vm) {
                    var guestLI = kimchi.createGuestLi(vm, currentConsoleImages[vm.name], vm.name==openMenuGuest);
                    $('#guestList').append(guestLI);
                });
            } else {
                $('#guestListField').hide();
                $('#noGuests').show();
            }
        }

        kimchi.vmTimeout = window.setTimeout("kimchi.listVmsAuto();", 5000);
    }, function(errorResponse, textStatus, errorThrown) {
        if(errorResponse.responseJSON && errorResponse.responseJSON.reason) {
            kimchi.message.error(errorResponse.responseJSON.reason);
        }
        kimchi.vmTimeout = window.setTimeout("kimchi.listVmsAuto();", 5000);
    });
};

kimchi.createGuestLi = function(vmObject, prevScreenImage, openMenu) {
    var result=kimchi.guestElem.clone();

    //Setup the VM list entry
    var vmRunningBool=(vmObject.state=="running");
    result.attr('id',vmObject.name);
    result.data(vmObject);

    //Add the Name
    var guestTitle=result.find('.title').attr('title',vmObject.name);
    guestTitle.html(vmObject.name);

    //Setup the VM console thumbnail display
    var curImg = vmObject.screenshot || vmObject.icon;
    var load_src = curImg || 'images/icon-vm.png';
    var tile_src = prevScreenImage || vmObject['load-src'];
    var liveTile=result.find('div[name=guest-tile] > .tile');
    liveTile.addClass(vmObject.state);
    liveTile.find('.imgactive').attr('src',tile_src);
    var imgLoad=liveTile.find('.imgload');
    imgLoad.on('load', function() {
                                     var oldImg=$(this).parent().find('.imgactive');
                                     oldImg.removeClass("imgactive").addClass("imgload");
                                     oldImg.attr("src","");
                                     $(this).addClass("imgactive").removeClass("imgload");
                                     $(this).off('load');
                                   });
    imgLoad.attr('src',load_src);

    //Link the stopped tile to the start action, the running tile to open the console
    if (vmRunningBool) {
        liveTile.off("click", kimchi.vmstart);
        liveTile.on("click", kimchi.openVmConsole);
    }
    else {
        liveTile.off("click", kimchi.openVmConsole);
        liveTile.on("click", kimchi.vmstart);
        liveTile.hover(function(event){$(this).find('.overlay').show()}, function(event){$(this).find('.overlay').hide()});
    }


    //Setup the gauges
    var stats=vmObject.stats;
    var gaugeValue=0;
    gaugeValue=parseInt(stats.net_throughput);
    kimchi.circleGaugeInit(result, "net_throughput",gaugeValue,(gaugeValue*100/stats.net_throughput_peak));
    gaugeValue=parseInt(stats.io_throughput);
    kimchi.circleGaugeInit(result, "io_throughput",gaugeValue,(gaugeValue*100/stats.io_throughput_peak));
    gaugeValue=parseInt(stats.cpu_utilization);
    kimchi.circleGaugeInit(result, "cpu_utilization",gaugeValue+"%",gaugeValue);

    //Setup the VM Actions
    var guestActions=result.find("div[name=guest-actions]");
    guestActions.find(".shutoff-disabled").prop('disabled', !vmRunningBool );
    guestActions.find(".running-disabled").prop('disabled', vmRunningBool );

    if (vmRunningBool) {
        guestActions.find(".running-hidden").hide();
    }
    else {
        guestActions.find(".shutoff-hidden").hide();
    }

    var consoleActions=guestActions.find("[name=vm-console]");

    if ((vmObject.graphics['type'] == 'vnc') || (vmObject.graphics['type'] == 'spice')) {
        consoleActions.on("click", kimchi.openVmConsole);
        consoleActions.show();
    } else {         //we don't recognize the VMs supported graphics, so hide the menu choice
        consoleActions.hide();
        consoleActions.off("click",kimchi.openVmConsole);
    }

    //Setup action event handlers
    guestActions.find("[name=vm-start]").on({click : kimchi.vmstart});
    guestActions.find("[name=vm-stop]").on({click : kimchi.vmstop});
    if (vmRunningBool) {  //If the guest is not running, do not enable reset
        guestActions.find("[name=vm-reset]").on({click : kimchi.vmreset});
    }
    guestActions.find("[name=vm-media]").on({click : kimchi.vmmedia});
    guestActions.find("[name=vm-edit]").on({click : kimchi.vmedit});
    guestActions.find("[name=vm-delete]").on({click : kimchi.vmdelete});

    //Maintain menu open state
    var actionMenu=guestActions.find("div[name=actionmenu]");
    if (openMenu) {
      actionMenu.addClass("open");
    }

    return result;
};

kimchi.circleGaugeInit = function(topElement, divName, display, percentage){
    var gauge=topElement.find('div[name="' + divName + '"] .circleGauge');
    if(gauge) {
        var data=Object();
        data.percentage = percentage;
        data.display = display;
        gauge.data(data);
    }
    gauge.circleGauge();
    return(gauge);
};

kimchi.guestSetRequestHeader = function(xhr) {
    xhr.setRequestHeader('Accept', 'text/html');
};

kimchi.guest_main = function() {
    $("#vm-add").on("click", function(event) {
        kimchi.window.open('guest-add.html');
    });
    kimchi.guestTemplate = $('#guest-tmpl').html();
    kimchi.guestElem=$('<div/>').html(kimchi.guestTemplate).find('li');
    $('#guests-root-container').on('remove', function() {
        kimchi.vmTimeout && clearTimeout(kimchi.vmTimeout);
    });
    kimchi.listVmsAuto()
};

kimchi.editTemplate = function(guestTemplate, oldPopStat) {
    if (oldPopStat) {
        return guestTemplate.replace("vm-action", "vm-action open");
    }
    return guestTemplate;
};
/*
 * Project Kimchi
 *
 * Copyright IBM, Corp. 2013
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
kimchi.guest_media_main = function() {

    var refreshCDROMs = function() {
        kimchi.listVMStorages({
            vm: kimchi.selectedGuest,
            storageType: 'cdrom'
        }, function(storages) {
            var rowHTML = $('#cdrom-row-tmpl').html();
            var container = $('#guest-edit-cdrom-row-container');
            $(container).empty();

            $.each(storages, function(index, storage) {
                storage['vm'] = kimchi.selectedGuest;
                var templated = kimchi.template(rowHTML, storage);
                container.append(templated);
            });

            var replaceCDROM = function(event) {
                event.preventDefault();
                kimchi.selectedGuestStorage = $(this).data('dev');
                kimchi.window.open("guest-cdrom-edit.html");
            };

            $('input[type="text"][name="cdrom"]', container).on('click', replaceCDROM);
            $('.replace', container).on('click', replaceCDROM);
        });
    };

    refreshCDROMs();

    var onReplaced = function(params) {
        refreshCDROMs();
    };
    kimchi.topic('kimchi/vmCDROMReplaced').subscribe(onReplaced);

    kimchi.clearGuestMedia = function() {
        kimchi.topic('kimchi/vmCDROMReplaced').unsubscribe(onReplaced);
    };
};
/*
 * Project Kimchi
 *
 * Copyright IBM, Corp. 2013
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
kimchi.guest_storage_add_main = function() {
    var types = [{
        label: 'cdrom',
        value: 'cdrom'
    }];
    kimchi.select('guest-storage-type-list', types);

    var storageAddForm = $('#form-guest-storage-add');
    var submitButton = $('#guest-storage-button-add');
    var nameTextbox = $('input[name="dev"]', storageAddForm);
    var typeTextbox = $('input[name="type"]', storageAddForm);
    var pathTextbox = $('input[name="path"]', storageAddForm);

    var submitForm = function(event) {
        if(submitButton.prop('disabled')) {
            return false;
        }

        var dev = nameTextbox.val();
        var type = typeTextbox.val();
        var path = pathTextbox.val();
        if(!path || path === '') {
            return false;
        }

        var formData = storageAddForm.serializeObject();
        $.each([submitButton, nameTextbox, pathTextbox], function(i, c) {
            $(c).prop('disabled', true);
        });
        $(submitButton).addClass('loading').text(i18n['KCHVMCD6003M']);

        var settings = {
            vm: kimchi.selectedGuest,
            type: type,
            path: path
        };

        if(dev && dev !== '') {
            settings['dev'] = dev;
        }

        kimchi.addVMStorage(settings, function(result) {
            kimchi.window.close();
            kimchi.topic('kimchi/vmCDROMAttached').publish({
                result: result
            });
        }, function(result) {
            var errText = result['reason'] ||
                result['responseJSON']['reason'];
            kimchi.message.error(errText);

            $.each([submitButton, nameTextbox, pathTextbox], function(i, c) {
                $(c).prop('disabled', false);
            });
            $(submitButton).removeClass('loading').text(i18n['KCHVMCD6002M']);
        });

        event.preventDefault();
    };

    storageAddForm.on('submit', submitForm);
    submitButton.on('click', submitForm);
    pathTextbox.on('input propertychange', function(event) {
        $(submitButton).prop('disabled', $(this).val() === '');
    });
};
/*
 * Project Kimchi
 *
 * Copyright IBM, Corp. 2013
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
kimchi.host={};

kimchi.host_main = function() {
    var expand = function(header, toExpand) {
        var controlledNode = $(header).attr('aria-controls');
        $('#' + controlledNode)[toExpand ? 'removeClass' : 'addClass']('hidden');
        $(header).attr('aria-expanded', toExpand ? 'true' : 'false');
    };

    var repositoriesGrid = null;
    var initRepositoriesGrid = function(repo_type) {
        var gridFields=[];
        if (repo_type == "yum") {
                gridFields=[{
                    name: 'repo_id',
                    label: i18n['KCHREPO6004M'],
                    'class': 'repository-id'
                }, {
                    name: 'config[repo_name]',
                    label: i18n['KCHREPO6005M'],
                    'class': 'repository-name'
                }, {
                    name: 'enabled',
                    label: i18n['KCHREPO6009M'],
                    'class': 'repository-enabled'
                }];
        }
        else if (repo_type == "deb") {
                gridFields=[{
                    name: 'baseurl',
                    label: i18n['KCHREPO6006M'],
                    makeTitle: true,
                    'class': 'repository-baseurl'
                }, {
                    name: 'enabled',
                    label: i18n['KCHREPO6009M'],
                    'class': 'repository-enabled'
                }, {
                    name: 'config[dist]',
                    label: "dist",
                    'class': 'repository-gpgcheck'
                }, {
                    name: 'config[comps]',
                    label: "comps",
                    'class': 'repository-gpgcheck'
                }];
        }
        else {
            gridFields=[{
                name: 'repo_id',
                label: i18n['KCHREPO6004M'],
                'class': 'repository-id'
                }, {
                    name: 'enabled',
                    label: i18n['KCHREPO6009M'],
                    'class': 'repository-enabled'
                }, {
                    name: 'baseurl',
                    label: i18n['KCHREPO6006M'],
                    makeTitle: true,
                    'class': 'repository-baseurl'
                }];
        }
        repositoriesGrid = new kimchi.widget.Grid({
            container: 'repositories-grid-container',
            id: 'repositories-grid',
            title: i18n['KCHREPO6003M'],
            toolbarButtons: [{
                id: 'repositories-grid-add-button',
                label: i18n['KCHREPO6012M'],
                onClick: function(event) {
                    kimchi.window.open({url:'repository-add.html', class: repo_type});
                }
            }, {
                id: 'repositories-grid-enable-button',
                label: i18n['KCHREPO6016M'],
                disabled: true,
                onClick: function(event) {
                    var repository = repositoriesGrid.getSelected();
                    if(!repository) {
                        return;
                    }
                    var name = repository['repo_id'];
                    var enable = !repository['enabled'];
                    $(this).prop('disabled', true);
                    kimchi.enableRepository(name, enable, function() {
                        kimchi.topic('kimchi/repositoryUpdated').publish();
                    });
                }
            }, {
                id: 'repositories-grid-edit-button',
                label: i18n['KCHREPO6013M'],
                disabled: true,
                onClick: function(event) {
                    var repository = repositoriesGrid.getSelected();
                    if(!repository) {
                        return;
                    }
                    kimchi.selectedRepository = repository['repo_id'];
                    kimchi.window.open({url:'repository-edit.html', class: repo_type});
                }
            }, {
                id: 'repositories-grid-remove-button',
                label: i18n['KCHREPO6014M'],
                disabled: true,
                onClick: function(event) {
                    var repository = repositoriesGrid.getSelected();
                    if(!repository) {
                        return;
                    }

                    var settings = {
                        title : i18n['KCHREPO6001M'],
                        content : i18n['KCHREPO6002M'],
                        confirm : i18n['KCHAPI6004M'],
                        cancel : i18n['KCHAPI6003M']
                    };

                    kimchi.confirm(settings, function() {
                        kimchi.deleteRepository(
                            repository['repo_id'],
                            function(result) {
                                kimchi.topic('kimchi/repositoryDeleted').publish(result);
                            }, function(error) {
                            }
                        );
                    });
                }
            }],
            onRowSelected: function(row) {
                var repository = repositoriesGrid.getSelected();
                if(!repository) {
                    return;
                }
                $('#repositories-grid-remove-button').prop('disabled', false);
                $('#repositories-grid-edit-button').prop('disabled', false);
                var enabled = repository['enabled'];
                $('#repositories-grid-enable-button')
                    .text(i18n[enabled ? 'KCHREPO6017M' : 'KCHREPO6016M'])
                    .prop('disabled', false);
            },
            frozenFields: [],
            fields: gridFields,
            data: listRepositories
        });
    };

    var listRepositories = function(gridCallback) {
        kimchi.listRepositories(function(repositories) {
            if($.isFunction(gridCallback)) {
                gridCallback(repositories);
            }
            else {
                if(repositoriesGrid) {
                    repositoriesGrid.setData(repositories);
                }
                else {
                    initRepositoriesGrid();
                    repositoriesGrid.setData(repositories);
                }
            }
        },
        function(error) {
            var message = error && error['responseJSON'] && error['responseJSON']['reason'];

            if($.isFunction(gridCallback)) {
                gridCallback([]);
            }
            repositoriesGrid &&
                repositoriesGrid.showMessage(message || i18n['KCHUPD6008M']);
        });

        $('#repositories-grid-remove-button').prop('disabled', true);
        $('#repositories-grid-edit-button').prop('disabled', true);
        $('#repositories-grid-enable-button').prop('disabled', true);
    };

    var softwareUpdatesGridID = 'software-updates-grid';
    var softwareUpdatesGrid = null;
    var progressAreaID = 'software-updates-progress-textarea';
    var reloadProgressArea = function(result) {
        var progressArea = $('#' + progressAreaID)[0];
        $(progressArea).text(result['message']);
        var scrollTop = $(progressArea).prop('scrollHeight');
        $(progressArea).prop('scrollTop', scrollTop);
    };

    var initSoftwareUpdatesGrid = function(softwareUpdates) {
        softwareUpdatesGrid = new kimchi.widget.Grid({
            container: 'software-updates-grid-container',
            id: softwareUpdatesGridID,
            title: i18n['KCHUPD6001M'],
            rowSelection: 'disabled',
            toolbarButtons: [{
                id: softwareUpdatesGridID + '-update-button',
                label: i18n['KCHUPD6006M'],
                disabled: true,
                onClick: function(event) {
                    var updateButton = $(this);
                    var progressArea = $('#' + progressAreaID)[0];
                    $('#software-updates-progress-container').removeClass('hidden');
                    $(progressArea).text('');
                    !kimchi.isElementInViewport(progressArea) &&
                        progressArea.scrollIntoView();
                    $(updateButton).text(i18n['KCHUPD6007M']).prop('disabled', true);

                    kimchi.updateSoftware(function(result) {
                        reloadProgressArea(result);
                        $(updateButton).text(i18n['KCHUPD6006M']).prop('disabled', false);
                        kimchi.topic('kimchi/softwareUpdated').publish({
                            result: result
                        });
                    }, function(error) {
                        var message = error && error['responseJSON'] && error['responseJSON']['reason'];
                        kimchi.message.error(message || i18n['KCHUPD6009M']);
                        $(updateButton).text(i18n['KCHUPD6006M']).prop('disabled', false);
                    }, reloadProgressArea);
                }
            }],
            frozenFields: [],
            fields: [{
                name: 'package_name',
                label: i18n['KCHUPD6002M'],
                'class': 'software-update-name'
            }, {
                name: 'version',
                label: i18n['KCHUPD6003M'],
                'class': 'software-update-version'
            }, {
                name: 'arch',
                label: i18n['KCHUPD6004M'],
                'class': 'software-update-arch'
            }, {
                name: 'repository',
                label: i18n['KCHUPD6005M'],
                'class': 'software-update-repos'
            }],
            data: listSoftwareUpdates
        });
    };

    var listSoftwareUpdates = function(gridCallback) {
        kimchi.listSoftwareUpdates(function(softwareUpdates) {
            if($.isFunction(gridCallback)) {
                gridCallback(softwareUpdates);
            }
            else {
                if(softwareUpdatesGrid) {
                    softwareUpdatesGrid.setData(softwareUpdates);
                }
                else {
                    initSoftwareUpdatesGrid(softwareUpdates);
                }
            }

            var updateButton = $('#' + softwareUpdatesGridID + '-update-button');
            $(updateButton).prop('disabled', softwareUpdates.length === 0);
        }, function(error) {
            var message = error && error['responseJSON'] && error['responseJSON']['reason'];
            if($.isFunction(gridCallback)) {
                gridCallback([]);
            }
            softwareUpdatesGrid &&
                softwareUpdatesGrid.showMessage(message || i18n['KCHUPD6008M']);
        });
    };

    var reportGridID = 'available-reports-grid';
    var reportGrid = null;
    var initReportGrid = function(reports) {
        reportGrid = new kimchi.widget.Grid({
            container: 'available-reports-grid-container',
            id: reportGridID,
            title: i18n['KCHDR6002M'],
            toolbarButtons: [{
                id: reportGridID + '-generate-button',
                label: i18n['KCHDR6006M'],
                onClick: function(event) {
                    kimchi.window.open('report-add.html', {
                        close: function() {
                            kimchi.stopTrackingReport = true;
                        }
                    });
                }
            }, {
                id: reportGridID + '-rename-button',
                label: i18n['KCHDR6008M'],
                disabled: true,
                onClick: function(event) {
                }
            }, {
                id: reportGridID + '-remove-button',
                label: i18n['KCHDR6009M'],
                disabled: true,
                onClick: function(event) {
                    var report = reportGrid.getSelected();
                    if(!report) {
                        return;
                    }

                    var settings = {
                        title : i18n['KCHAPI6004M'],
                        content : i18n['KCHDR6001M'],
                        confirm : i18n['KCHAPI6002M'],
                        cancel : i18n['KCHAPI6003M']
                    };

                    kimchi.confirm(settings, function() {
                        kimchi.deleteReport({
                            name: report['name']
                        }, function(result) {
                            $('#' + reportGridID + '-remove-button')
                                .prop('disabled', true);
                            $('#' + reportGridID + '-download-button')
                                .prop('disabled', true);
                            listDebugReports();
                        }, function(error) {
                           kimchi.message.error(error.responseJSON.reason);
                        });
                    });
                }
            }, {
                id: reportGridID + '-download-button',
                label: i18n['KCHDR6010M'],
                disabled: true,
                onClick: function(event) {
                    var report = reportGrid.getSelected();
                    if(!report) {
                        return;
                    }

                    kimchi.downloadReport({
                        file: report['uri']
                    });
                }
            }],
            onRowSelected: function(row) {
                $('#' + reportGridID + '-remove-button')
                    .prop('disabled', false);
                $('#' + reportGridID + '-download-button')
                    .prop('disabled', false);
            },
            frozenFields: [{
                name: 'id',
                label: ' ',
                'class': 'debug-report-id'
            }],
            fields: [{
                name: 'name',
                label: i18n['KCHDR6003M'],
                'class': 'debug-report-name'
            }, {
                name: 'time',
                label: i18n['KCHDR6005M'],
                'class': 'debug-report-time'
            }],
            data: reports
        });
    };

    var listDebugReports = function() {
        kimchi.listReports(function(reports) {
            $('#debug-report-section').removeClass('hidden');
            $.each(reports, function(i, item) {
                reports[i]['id'] = i + 1;
            });
            if(reportGrid) {
                reportGrid.setData(reports);
            }
            else {
                initReportGrid(reports);
            }
        }, function(error) {
            if(error['status'] == 403) {
                $('#debug-report-section').addClass('hidden');
                return;
            }
            $('#debug-report-section').removeClass('hidden');
        });
    };

    var shutdownButtonID = '#host-button-shutdown';
    var restartButtonID = '#host-button-restart';
    var shutdownHost = function(params) {
        var settings = {
            title : i18n['KCHAPI6004M'],
            content : i18n['KCHHOST6008M'],
            confirm : i18n['KCHAPI6002M'],
            cancel : i18n['KCHAPI6003M']
        };

        kimchi.confirm(settings, function() {
            kimchi.shutdown(params);
            $(shutdownButtonID).prop('disabled', true);
            $(restartButtonID).prop('disabled', true);
            // Check if there is any VM is running.
            kimchi.listVMs(function(vms) {
                for(var i = 0; i < vms.length; i++) {
                    if(vms[i]['state'] === 'running') {
                        kimchi.message.error.code('KCHHOST6001E');
                        $(shutdownButtonID).prop('disabled', false);
                        $(restartButtonID).prop('disabled', false);
                        return;
                    }
                }

            });
        }, function() {
        });
    };

    var initPage = function() {
        $('#host-info-container .section-header').each(function(i, header) {
            $('<span class="arrow"></span>').prependTo(header);
            var toExpand = $(header).attr('aria-expanded') !== 'false';
            expand(header, toExpand);
        });

        $('#host-info-container').on('click', '.section-header', function(event) {
            var toExpand = $(this).attr('aria-expanded') === 'false';
            expand(this, toExpand);
        });

        $('#host-button-shutdown').on('click', function(event) {
            shutdownHost(null);
        });

        $('#host-button-restart').on('click', function(event) {
            shutdownHost({
                reboot: true
            });
        });

        var keepMonitoringCheckbox = $('#keep-monitoring-checkbox');
        keepMonitoringCheckbox.prop('checked', kimchi.keepMonitoringHost === true);
        keepMonitoringCheckbox.on('change', function(event) {
            kimchi.keepMonitoringHost = this['checked'];
        });

        kimchi.getCapabilities(function(capabilities) {
            kimchi.host.capabilities=capabilities;
            if((capabilities['repo_mngt_tool']) && (capabilities['repo_mngt_tool']!="None")) {
                initRepositoriesGrid(capabilities['repo_mngt_tool']);
                $('#repositories-section').switchClass('hidden', capabilities['repo_mngt_tool']);
                kimchi.topic('kimchi/repositoryAdded')
                    .subscribe(listRepositories);
                kimchi.topic('kimchi/repositoryUpdated')
                    .subscribe(listRepositories);
                kimchi.topic('kimchi/repositoryDeleted')
                    .subscribe(listRepositories);
            }

            if(capabilities['update_tool']) {
                $('#software-update-section').removeClass('hidden');
                initSoftwareUpdatesGrid();
                kimchi.topic('kimchi/softwareUpdated')
                    .subscribe(listSoftwareUpdates);
            }

            if(capabilities['system_report_tool']) {
                listDebugReports();
                kimchi.topic('kimchi/debugReportAdded')
                    .subscribe(listDebugReports);
            }
        });
    };

    kimchi.getHost(function(data) {
        var htmlTmpl = $('#host-tmpl').html();
        data['logo'] = data['logo'] || '';
        data['memory'] = kimchi.formatMeasurement(data['memory'], {
            fixed: 2
        });
        var templated = kimchi.template(htmlTmpl, data);
        $('#host-content-container').html(templated);

        initPage();
        initTracker();
    });

    var StatsMgr = function() {
        var statsArray = {
            cpu: {
                u: {
                    type: 'percent',
                    legend: i18n['KCHHOST6002M'],
                    points: []
                }
            },
            memory: {
                u: {
                    type: 'value',
                    base: 2,
                    fixed: 2,
                    legend: i18n['KCHHOST6003M'],
                    points: []
                }
            },
            diskIO: {
                r: {
                    type: 'value',
                    base: 2,
                    fixed: 2,
                    unit: 'B/s',
                    legend: i18n['KCHHOST6004M'],
                    points: []
                },
                w: {
                    type: 'value',
                    base: 2,
                    fixed: 2,
                    unit: 'B/s',
                    legend: i18n['KCHHOST6005M'],
                    'class': 'disk-write',
                    points: []
                }
            },
            networkIO: {
                r: {
                    type: 'value',
                    base: 2,
                    fixed: 2,
                    unit: 'B/s',
                    legend: i18n['KCHHOST6006M'],
                    points: []
                },
                s: {
                    type: 'value',
                    base: 2,
                    fixed: 2,
                    unit: 'B/s',
                    legend: i18n['KCHHOST6007M'],
                    'class': 'network-sent',
                    points: []
                }
            }
        };
        var SIZE = 20;
        var cursor = SIZE;

        var add = function(stats) {
            for(var key in stats) {
                var item = stats[key];
                for(var metrics in item) {
                    var value = item[metrics]['v'];
                    var max = item[metrics]['max'];
                    var unifiedMetrics = statsArray[key][metrics];
                    var ps = unifiedMetrics['points'];
                    ps.push(value);
                    ps.length > SIZE + 1 &&
                        ps.shift();
                    if(max !== undefined) {
                        unifiedMetrics['max'] = max;
                    }
                    else {
                        if(unifiedMetrics['type'] !== 'value') {
                            continue;
                        }
                        max = -Infinity;
                        $.each(ps, function(i, value) {
                            if(value > max) {
                                max = value;
                            }
                        });
                        if(max === 0) {
                            ++max;
                        }
                        max *= 1.1;
                        unifiedMetrics['max'] = max;
                    }
                }
            }
            cursor++;
        };

        var get = function(which) {
            var stats = statsArray[which];
            var lines = [];
            for(var k in stats) {
                var obj = stats[k];
                var line = {
                    type: obj['type'],
                    base: obj['base'],
                    unit: obj['unit'],
                    fixed: obj['fixed'],
                    legend: obj['legend']
                };
                if(obj['max']) {
                    line['max'] = obj['max'];
                }
                if(obj['class']) {
                    line['class'] = obj['class'];
                }
                var ps = obj['points'];
                var numStats = ps.length;
                var unifiedPoints = [];
                $.each(ps, function(i, value) {
                    unifiedPoints.push({
                        x: cursor - numStats + i,
                        y: value
                    });
                });
                line['points'] = unifiedPoints;
                lines.push(line);
            }
            return lines;
        };

        return {
            add: add,
            get: get
        };
    };

    var Tracker = function(charts) {
      var charts = charts;
      var timer = null;
      var statsPool = new StatsMgr();
      var setCharts = function(newCharts) {
          charts = newCharts;
          for(var key in charts) {
              var chart = charts[key];
              chart.updateUI(statsPool.get(key));
          }
      };

      var self = this;
      var track = function() {
          kimchi.getHostStats(function(stats) {
              var unifiedStats = {
                  cpu: {
                      u: {
                          v: stats['cpu_utilization']
                      }
                  },
                  memory: {
                      u: {
                          v: stats['memory']['avail'],
                          max: stats['memory']['total']
                      }
                  },
                  diskIO: {
                      r: {
                          v: stats['disk_read_rate']
                      },
                      w: {
                          v: stats['disk_write_rate']
                      }
                  },
                  networkIO: {
                      r: {
                          v: stats['net_recv_rate']
                      },
                      s: {
                          v: stats['net_sent_rate']
                      }
                  }
              };
              statsPool.add(unifiedStats);
              for(var key in charts) {
                  var chart = charts[key];
                  chart.updateUI(statsPool.get(key));
              }
              timer = setTimeout(function() {
                  track();
              }, 1000);
          }, function() {
          });
      };

      var destroy = function() {
          timer && clearTimeout(timer);
          timer = null;
      };

      return {
        setCharts: setCharts,
        start: track,
        stop: destroy
      };
    };

    var initTracker = function() {
        // TODO: Extend tabs with onUnload event to unregister timers.
        if(!kimchi.keepMonitoringHost && kimchi.hostTimer) {
            var timer = kimchi.hostTimer;
            timer.stop();
            timer = null;
            kimchi.hostTimer = null;
            delete kimchi.hostTimer;
        }

        var trackedCharts = {
            cpu: new kimchi.widget.LineChart({
                id: 'chart-cpu',
                node: 'container-chart-cpu',
                type: 'percent'
            }),
            memory: new kimchi.widget.LineChart({
                id: 'chart-memory',
                node: 'container-chart-memory',
                type: 'value'
            }),
            diskIO: new kimchi.widget.LineChart({
                id: 'chart-disk-io',
                node: 'container-chart-disk-io',
                type: 'value'
            }),
            networkIO: new kimchi.widget.LineChart({
                id: 'chart-network-io',
                node: 'container-chart-network-io',
                type: 'value'
            })
        };

        if(kimchi.hostTimer) {
            kimchi.hostTimer.setCharts(trackedCharts);
        }
        else {
            kimchi.hostTimer = new Tracker(trackedCharts);
            kimchi.hostTimer.start();
        }
    };

    $('#host-root-container').on('remove', function() {
        if(!kimchi.keepMonitoringHost && kimchi.hostTimer) {
            kimchi.hostTimer.stop();
            kimchi.hostTimer = null;
            kimchi.hostTimer = null;
            delete kimchi.hostTimer;
        }

        repositoriesGrid && repositoriesGrid.destroy();
        kimchi.topic('kimchi/repositoryAdded')
            .unsubscribe(listRepositories);
        kimchi.topic('kimchi/repositoryUpdated')
            .unsubscribe(listRepositories);
        kimchi.topic('kimchi/repositoryDeleted')
            .unsubscribe(listRepositories);

        softwareUpdatesGrid && softwareUpdatesGrid.destroy();
        kimchi.topic('kimchi/softwareUpdated').unsubscribe(listSoftwareUpdates);

        reportGrid && reportGrid.destroy();
        kimchi.topic('kimchi/debugReportAdded').unsubscribe(listDebugReports);
    });
};
/*
 * Project Kimchi
 *
 * Copyright IBM, Corp. 2013
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
kimchi.lang = {
    all: function() {
        return {
          'en_US': 'English (US)',
          'zh_CN': '中文（简体）',
          'pt_BR': 'Português (Brasil)',
          'de_DE': 'Deutsch (Deutschland)',
          'es_ES': 'Español (España)',
          'fr_FR': 'Français (France)',
          'it_IT': 'Italiano (Italia)',
          'ja_JP': '日本語 (日本)',
          'ko_KR': '한국어 (대한민국)',
          'ru_RU': 'Русский (Россия)',
          'zh_TW': '中文（繁體）'
        };
    },

    /**
     * Language is determined by the following sequence:
     * 1) Cookie setting; or if not set ->
     * 2) HTML DOM lang attribute; or if not set ->
     * 3) DEFAULT (en_US).
     */
    get: function() {
        return kimchi.cookie.get('kimchiLang') ||
            $('html').prop('lang') ||
            'en_US';
    },

    set: function(lang) {
        kimchi.cookie.set('kimchiLang', lang, 365);
    }
};
/*
 * Project Kimchi
 *
 * Copyright IBM, Corp. 2013
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/**
 * new kimchi.widget.LineChart({
 *   node: 'line-chart-cpu',
 *   id: 'line-chart',
 *   type: 'value'
 * });
 */
kimchi.widget.LineChart = function(params) {
    var container = $('#' + params['node']);
    container.addClass('chart-container');
    var height = container.height();
    var width = container.width();
    var numHLines = 4;
    var linesSpace = height / numHLines;
    var period = params['period'] || 20;
    var xFactor = width / period;
    var yFactor = height / 100;
    var xStart = params['xStart'] || 0;
    var linesOffset = 0;
    var canvasID = params['id'];
    var maxValue = params['maxValue'] || -Infinity;
    var type = params['type'];
    var chartVAxis = null;
    var chartTitle = null;
    var chartLegend = null;
    var seriesMap = {};
    var formatSettings = {};

    var setMaxValue = function(newValue) {
        maxValue = newValue;
    };

    /**
     *
     * settings: {
     *   'class': 'disk-read-rate'
     * }
     */
    var updateUI = function(data) {
        var container = $('#' + params['node']);
        if(!container.length) {
            return;
        }

        if(!$.isArray(data)) {
            data = [data];
        }
        var seriesCount = 0;
        var singleSeries = data.length === 1;
        var firstSeries = data[0];

        // TODO: Multiple axes support.
        if(type === 'value') {
            $.each(data, function(i, series) {
                if(series['max'] > maxValue) {
                    maxValue = series['max'];
                    formatSettings = {
                        base: series['base'],
                        unit: series['unit'],
                        fixed: series['fixed']
                    };
                }
            });
        }

        var canvasNode = $('#' + canvasID);
        canvasNode.length && canvasNode.remove();
        var htmlStr = [
          '<svg id="', canvasID, '" class="line-chart"',
              ' height="', height, '" width="', width, '"',
          '>',
            '<rect height="', height, '" width="', width, '" class="background" />'
        ];

        for(var x = linesOffset; x < width; x += linesSpace) {
            htmlStr.push(
                '<line x1="', x, '" y1="', 0, '" x2="', x, '" y2="', height, '" />'
            );
        }

        linesOffset -= xFactor;
        while(linesOffset < 0) {
            linesOffset = linesSpace + linesOffset;
        }

        for(var y = height - linesSpace; y > 0; y -= linesSpace) {
            htmlStr.push(
                '<line x1="', 0, '" y1="', y, '" x2="', width, '" y2="', y, '" />'
            );
        }

        var maxValueLabel = i18n['KCHHOST6001M'] + ' ' +
            (type === 'value'
                ? kimchi.formatMeasurement(maxValue, formatSettings)
                : '100%');
        if(!chartVAxis) {
            chartVAxis = $('<div class="chart-vaxis-container">' +
                maxValueLabel +
                '</div>'
            );
            container.before(chartVAxis);
        }
        else {
            chartVAxis.text(maxValueLabel);
        }

        seriesNames = [];
        $.each(data, function(i, series) {
            var points = series['points'];
            var className = series['class'];
            var latestPoint = points.slice(-1).pop();
            xStart = latestPoint['x'] - period;

            htmlStr.push('<polyline',
                ' class="series', className ? ' ' + className : '', '"',
                ' points="'
            );
            var first = true;
            $.each(points, function(i, point) {
                if(first) {
                    first = false;
                }
                else {
                    htmlStr.push(' ');
                }

                var x = xFactor * (point['x'] - xStart);
                var y = height - yFactor * (type === 'value' ?
                    point['y'] * 100 / maxValue :
                    point['y']
                );
                htmlStr.push(x, ',', y);
            });
            htmlStr.push('" />');
        });

        htmlStr.push('</svg>');

        var canvasNode = $(htmlStr.join('')).appendTo(container);

        if(!chartLegend) {
            chartLegend = $('<div class="chart-legend-container"></div>');
            container.after(chartLegend);
        }
        else {
            chartLegend.empty();
        }
        $('polyline.series', canvasNode).each(function(i, polyline) {
            var wrapper = $('<div class="legend-wrapper"></div>')
                .appendTo(chartLegend);
            $([
                '<svg class="legend-icon" width="20" height="10">',
                    '<line x1="0" y1="5" x2="20" y2="5"/>',
                '</svg>'
            ].join('')).appendTo(wrapper);
            $('line', wrapper).css({
                stroke: $(polyline).css('stroke'),
                'stroke-width': $(polyline).css('stroke-width')
            });
            var label = data[i]['legend'];
            var base = data[i]['base'];
            $('<label class="legend-label">' + label + '</label>')
                .appendTo(wrapper);
            var latestPoint = data[i]['points'].slice(-1).pop();
            var latestValue = latestPoint['y'];
            if(type === 'value') {
                latestValue = kimchi.formatMeasurement(
                    latestValue,
                    formatSettings
                );
            }
            else {
                latestValue += '%';
            }
            $('<div class="latest-value">' + latestValue + '</div>')
                .appendTo(wrapper);
        });
    };

    return {
        setMaxValue: setMaxValue,
        updateUI: updateUI
    }
};
/*
 * Project Kimchi
 *
 * Copyright IBM, Corp. 2013
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
kimchi.login_main = function() {

    var selectedLanguage = kimchi.lang.get();
    var languages = kimchi.lang.all();
    for(var k in languages) {
        var opt = $([
            '<option value="',
            k,
            '">',
            languages[k],
            '</option>'
        ].join('')).appendTo($('#language'));
        $(opt).prop('selected', selectedLanguage === k);
    }

    $('#language').on('change', function() {
        kimchi.topic('languageChanged').publish($(this).val());
    });

    var validateNonEmpty = function(idsArray) {
        for(var i = 0; i < idsArray.length; i++) {
            var id = idsArray[i];
            if (!$('#' + id).val()) {
                $('#' + id + '-msg').text(i18n['KCHAUTH6002E']);
                placeCursor(id);
                return false;
            }
            else {
                $('#' + id + '-msg').empty();
            }
        }

        return true;
    };

    var placeCursor = function(id) {
        if (id && $('#' + id).size() > 0) {
            $('#' + id).focus();
            return;
        }

        var userName = kimchi.user.getUserID();
        userName && $('#user-id').val(userName);

        var nodeToFocus = ! $('#user-id').val() ? $('#user-id') :
            (! $('#password').val() ? $('#password') : $('#btn-login'));

        $(nodeToFocus).focus();
    };

    var login = function(event) {

        if (!validateNonEmpty(['user-id', 'password'])) {
            return false;
        }

        $('#btn-login').text(i18n['KCHAUTH6002M']).prop('disabled', true);

        var userID = $('#user-id').val();
        userID && kimchi.user.setUserID(userID);
        var settings = {
            userid: userID,
            password: $("#password").val()
        };

        kimchi.login(settings, function() {
            var pAjax = kimchi.previousAjax;
            var consoleURL = kimchi.cookie.get("console_uri");
            if (consoleURL) {
                var path = /.*\/(.*?)\?.*/g.exec(consoleURL)[1];
                var query = consoleURL.substr(consoleURL.indexOf("?") + 1);

                var proxy_port = /.*port=(.*?)(&|$)/g.exec(consoleURL)[1];
                var http_port = /.*kimchi=(.*?)(&|$)/g.exec(consoleURL);
                var kimchi_port = http_port ? http_port[1] : location.port;

                url = location.protocol + "//" + location.hostname;
                url += ":" + proxy_port + "/console.html?url=" + path;
                url += "&" + query;
                url += "&kimchi=" + kimchi_port;

                kimchi.cookie.remove("console_uri");
                window.location.replace(url)
            }
            else if (pAjax && true === pAjax['resend']) {
                pAjax['error'] = pAjax['originalError'];
                $.ajax(pAjax);
                kimchi.previousAjax = null;
            }
            else if(pAjax) {
                window.location.reload();
            }

            kimchi.user.showUser(true);
            kimchi.window.close();
        }, function() {
            kimchi.message.error.code('KCHAUTH6001E');
            $('#btn-login').prop('disabled', false).text(i18n['KCHAUTH6001M']);
            placeCursor('user-id');
        });

        return false;
    };

    $('#form-login').on('submit', login);

    placeCursor();
};
/*
 * Project Kimchi
 *
 * Copyright IBM, Corp. 2013
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
kimchi.main = function() {
    var tabUrl = "/config/ui/tabs.xml";
    var DEFAULT_HASH = kimchi.getDefaultPage(tabUrl);
    kimchi.popable();

    var genTabs = function(tabs) {
        var tabsHtml = [];
        $(tabs).each(function(i, tab) {
            var title = tab['title'];
            var path = tab['path'];
            tabsHtml.push(
                '<li>',
                    '<a class="item" href="', path, '">',
                        title,
                    '</a>',
                '</li>'
            );
        });
        return tabsHtml.join('');
    };

    var parseTabs = function(xmlData) {
        var tabs = [];
        $(xmlData).find('tab').each(function() {
            var $tab = $(this);
            var titleKey = $tab.find('title').text();
            var title = i18n[titleKey];
            var path = $tab.find('path').text();
            tabs.push({
                title: title,
                path: path
            });
        });

        return tabs;
    };

    var retrieveTabs = function(url) {
        var tabs;
        $.ajax({
            url : url,
            async : false,
            success : function(xmlData) {
                tabs = parseTabs(xmlData);
            }
        });
        return tabs;
    };

    var tabConfigUrl = '/config/ui/tabs.xml';
    var pluginConfigUrl = '/plugins/{plugin}/ui/config/tab-ext.xml';
    var DEFAULT_HASH;
    var buildTabs = function(callback) {
        var tabs = retrieveTabs(tabConfigUrl);
        kimchi.listPlugins(function(plugins) {
            $(plugins).each(function(i, p) {
                var url = kimchi.template(pluginConfigUrl, {
                    plugin: p
                });
                tabs.push.apply(tabs, retrieveTabs(url));
            });

            var firstTabPath = tabs[0] && tabs[0]['path'];
            DEFAULT_HASH = firstTabPath &&
                firstTabPath.substring(0, firstTabPath.length - 5);

            $('#nav-menu').append(genTabs(tabs));

            callback && callback();
        });
    };

    var onLanguageChanged = function(lang) {
        kimchi.lang.set(lang);
        location.reload();
    };

    /**
     * Do the following setup:
     *   1) Clear any timing events.
     *   2) If the given URL is invalid (i.e., no corresponding href value in
     *      page tab list.), then clear location.href and inform the user;
     *
     *      Or else:
     *      Move the page tab indicator to the right position;
     *      Load the page content via Ajax.
     */
    var onKimchiRedirect = function(url) {
        /*
         * Find the corresponding tab node and animate the arrow indicator to
         * point to the tab. If nothing found, inform user the URL is invalid
         * and clear location.hash to jump to home page.
         */
        var tab = $('#nav-menu a[href="' + url + '"]');
        if (tab.length === 0) {
            kimchi.message.error.code('KCHAPI6001E');
            location.hash = '';
            return;
        }

        // Animate arrow indicator.
        var left = $(tab).parent().position().left;
        var width = $(tab).parent().width();
        $('.menu-arrow').stop().animate({
            left : left + width / 2 - 10
        });

        // Update the visual style of tabs; focus the selected one.
        $('#nav-menu a').removeClass('current');
        $(tab).addClass('current');
        $(tab).focus();

        // Load page content.
        loadPage(url);
    };

    /**
     * Use Ajax to dynamically load a page without a page refreshing. Handle
     * arrow cursor animation, DOM node focus, and page content rendering.
     */
    var loadPage = function(url) {
        // Get the page content through Ajax and render it.
        url && $('#main').load(url, function(responseText, textStatus, jqXHR) {});
    };

    /*
     * Update page content.
     * 1) If user types in the main page URL without hash, then we apply the
     *    default hash. e.g., http://kimchi.company.com:8000;
     * 2) If user types a URL with hash, then we publish an "redirect" event
     *    to load the page, e.g., http://kimchi.company.com:8000/#templates.
     */
    var updatePage = function() {
        // Parse hash string.
        var hashString = (location.hash && location.hash.substr(1));
        /*
         * If hash string is empty, then apply the default one;
         * or else, publish an "redirect" event to load the page.
         */
        if (!hashString) {
            location.hash = DEFAULT_HASH;
        }
        else {
            kimchi.topic('redirect').publish(hashString + '.html');
        }
    };

    /**
     * Register listeners including:
     * 1) Kimchi redirect event
     * 2) hashchange event
     * 3) Tab list click event
     * 4) Log-out button click event
     */
    var initListeners = function() {
        kimchi.topic('languageChanged').subscribe(onLanguageChanged);
        kimchi.topic('redirect').subscribe(onKimchiRedirect);

        /*
         * If hash value is changed, then we know the user is intended to load
         * another page.
         */
        window.onhashchange = updatePage;

        /*
         * Register click listener of tabs. Replace the default reloading page
         * behavior of <a> with Ajax loading.
         */
        $('#nav-menu').on('click', 'a.item', function(event) {
            var href = $(this).attr('href');
            location.hash = href.substring(0,href.length -5);
            /*
             * We use the HTML file name for hash, like: guests for guests.html
             * and templates for templates.html.
             *     Retrieve hash value from the given URL and update location's
             * hash part. It has 2 effects: one is to publish Kimchi "redirect"
             * event to trigger listener, the other is to put an entry into the
             * browser's address history to make pages be bookmark-able.
             */
            // Prevent <a> causing browser redirecting to other page.
            event.preventDefault();
        });

        // Perform logging out via Ajax request.


        $('#btn-logout').on('click', function() {
            kimchi.logout(function() {
                updatePage();
            }, function(err) {
                kimchi.message.error(err.responseJSON.reason);
            });
        });
        $('#btn-about').on('click', function(event) {
            kimchi.window.open({"content": $('#about-tmpl').html()});
            event.preventDefault();
            });

        $('#btn-help').on('click', kimchi.getHelp);
    };

    // Load i18n translation strings first and then render the page.
    $('#main').load('i18n.html', function() {
        kimchi.addTabs(tabUrl);
        $(document).bind('ajaxError', function(event, jqXHR, ajaxSettings, errorThrown) {
            if (!ajaxSettings['kimchi']) {
                return;
            }

            if (jqXHR['status'] === 401) {
                kimchi.user.showUser(false);
                kimchi.previousAjax = ajaxSettings;
                $(".empty-when-logged-off").empty();
                $(".remove-when-logged-off").remove();
                kimchi.window.open({
                    url: 'login-window.html',
                    id: 'login-window-wrapper'
                });
                return;
            }
            else if((jqXHR['status'] == 0) && ("error"==jqXHR.statusText)) {
                kimchi.message.error(i18n['KCHAPI6007E'].replace("%1", jqXHR.state()));
            }
            if(ajaxSettings['originalError']) {
                ajaxSettings['originalError'](jqXHR, jqXHR.statusText, errorThrown);
            }
        });

        kimchi.user.showUser(true);
        initListeners();
        updatePage();
    });
};

kimchi.addTabs = function(url) {
    var tabsHtml = kimchi.getTabHtml(url);
    $('#nav-menu').prepend(tabsHtml);
    kimchi.addExtTabs();
};

kimchi.addExtTabs = function() {
    kimchi.listPlugins(function(results) {
        for ( var i = 0; i < results.length; i++) {
            var tabsHtml = kimchi.getTabHtml("/plugins/" + results[i] + "/ui/config/tab-ext.xml");
            $('#nav-menu').append(tabsHtml);
        }
    });
};

kimchi.getDefaultPage = function(url) {
    var defautLocation = "";
    $.ajax({
        url : url,
        async : false,
        success : function(xmlData) {
            var tab = $(xmlData).find('tab').first();
            var path = tab.find('path').text();
            if (path) {
                defautLocation = path.substring(0, path.length - 5);
            }
        }
    });
    return defautLocation;
};

kimchi.getTabHtml = function(url) {
    var tabsHtml = "";
    $.ajax({
        url : url,
        async : false,
        success : function(xmlData) {
            $(xmlData).find('tab').each(function() {
                var $tab = $(this);
                var titleKey = $tab.find('title').text();
                var title = i18n[titleKey];
                if(!title) title="Administration";
                var path = $tab.find('path').text();
                tabsHtml += "<li><a class='item' href=" + path + ">" + title + "</a></li>";
            });
        }
    });
    return tabsHtml;
};

kimchi.getHelp = function(e) {
        var url = window.location.hash;
        var lang = kimchi.lang.get();
        url = url.replace("#tabs", "/help/" + lang);
        if (url == "/help" + lang)
            url = url + "/index.html"
        else
            url = url + ".html";

        window.open(url, "Kimchi Help");
        e.preventDefault();
};
/*
 * Project Kimchi
 *
 * Copyright IBM, Corp. 2013
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
kimchi.message = function(msg, level, node) {
    var container = node || $('#messageField');
    if ($(container).size() < 1) {
        container = $('<div id="messageField"/>').appendTo(document.body);
    }
    var message = '<div class="message ' + (level || '') + '" style="display: none;">';
    if(!node) {
        message += '<div class="close">X</div>';
    }
    message += '<div class="content">' + msg + '</div>';
    message += '</div>';
    var $message = $(message);
    $(container).append($message);
    $message.fadeIn(100);

    setTimeout(function() {
        $message.fadeOut(2000, function() {
            $(this).remove();
        });
    }, 4000);

    $(container).on("click", ".close", function(e) {
        $(this).parent().fadeOut(200, function() {
            $(this).remove();
        });
    });
};

/**
 * A public function of confirm box.
 *
 * @param msg
 *            type:[object]
 * @param msg.title
 *            The title of the confirm box.
 * @param msg.content
 *            The main text of the confirm box.
 * @param msg.confirm
 *            The text of the confirm button.
 * @param msg.cancel
 *            the text of the cancel button.
 * @param confirmCallback
 *            the callback function of click the confirm button.
 * @param cancelCallback
 *            The callback function of click the cancel and X button.
 */
kimchi.confirm = function(settings, confirmCallback, cancelCallback) {
    if ($('#confirmbox-container ').size() < 1) {
        $(document.body).append('<div id="confirmbox-container" class="bgmask"></div>');
    }
    var confirmboxHtml = '<div class="confirmbox">';
    confirmboxHtml += '<header>';
    confirmboxHtml += '<h4 class="title">' + (settings.title || '') + '</h4>';
    confirmboxHtml += '<div class="close cancel">X</div>';
    confirmboxHtml += '</header>';
    confirmboxHtml += '<div class="content">';
    confirmboxHtml += settings.content + '</div>';
    confirmboxHtml += '<footer>';
    confirmboxHtml += '<div class="btn-group">';
    confirmboxHtml += '<button id="button-confirm" class="btn-small"><span class="text">' + (settings.confirm || i18n['KCHAPI6004M']) + '</span></button>';
    confirmboxHtml += '<button id="button-cancel" class="btn-small cancel"><span class="text">' + (settings.cancel || i18n['KCHAPI6003M']) + '</span></button>';
    confirmboxHtml += '</div>';
    confirmboxHtml += '</footer>';
    confirmboxHtml += '</div>';
    var confirmboxNode = $(confirmboxHtml);
    $('#confirmbox-container').append(confirmboxNode);
    confirmboxNode.fadeIn();

    $('#confirmbox-container').on("click", "#button-confirm", function(e) {
        if (confirmCallback) {
            confirmCallback();
        }
        confirmboxNode.fadeOut(1, function() {
            $('#confirmbox-container').remove();
        });
    });
    $('#confirmbox-container').on("click", ".cancel", function(e) {
        if (cancelCallback) {
            cancelCallback();
        }
        confirmboxNode.fadeOut(1, function() {
            $('#confirmbox-container').remove();
        });
    });
};

kimchi.message.warn = function(msg, node) {
    kimchi.message(msg, 'warn', node);
};
kimchi.message.error = function(msg, node) {
    kimchi.message(msg, 'error', node);
};
kimchi.message.error.code = function(code) {
    msg = code + ": " + i18n[code]
    kimchi.message(msg, 'error');
};
kimchi.message.success = function(msg, node) {
    kimchi.message(msg, 'success', node);
};
/*
 * Project Kimchi
 *
 * Copyright IBM, Corp. 2013
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

kimchi.NETWORK_TYPE_BRIDGE = "bridged";

kimchi.initNetwork = function() {
    kimchi.initNetworkListView();
    kimchi.initNetworkDialog();
    kimchi.initNetworkCreation();
    kimchi.initNetworkCleanup();
};

kimchi.initNetworkListView = function() {
    kimchi.listNetworks(function(data) {
        for (var i = 0; i < data.length; i++) {
            var network = {
                name : data[i].name,
                in_use : data[i].in_use,
                state : data[i].state === "active" ? "up" : "down"
            };
            if (data[i].connection === "bridge") {
                network.type = kimchi.NETWORK_TYPE_BRIDGE;
            } else {
                network.type = data[i].connection;
            }
            network.interface = data[i].interface ? data[i].interface : null;
            network.addrSpace = data[i].subnet ? data[i].subnet : null;
            kimchi.addNetworkItem(network);
        }
    });
};

kimchi.addNetworkItem = function(network) {
    $("#networkBody").append(kimchi.getNetworkItemHtml(network));
    kimchi.addNetworkActions(network);
};

kimchi.getNetworkItemHtml = function(network) {
    if(!network.interface) {
        network.interface = i18n["KCHNET6001M"];
    }
    if(!network.addrSpace) {
        network.addrSpace = i18n["KCHNET6001M"];
    }
    if(i18n["network_type_" + network.type]) {
        network.type = i18n["network_type_" + network.type];
    }

    var disable_in_use = network.in_use ? "ui-state-disabled" : "";
    var networkItem = kimchi.template($('#networkItem').html(), {
        name : network.name,
        state : network.state,
        type : network.type,
        interface: network.interface,
        addrSpace : network.addrSpace,
        startClass : network.state === "up" ? "hide-action-item" : "",
        stopClass : network.state === "down" ? "hide-action-item" : disable_in_use,
        stopDisabled : network.in_use ? "disabled" : "",
        deleteClass : network.state === "up" || network.in_use ? "ui-state-disabled" : "",
        deleteDisabled: network.state === "up" || network.in_use ? "disabled" : ""
    });
    return networkItem;
};

kimchi.addNetworkActions = function(network) {
    $(".menu-container", "#" + network.name).menu({
        position : {
            my : "left top",
            at : "left bottom",
            of : "#" + network.name
        },
        select : function(evt, ui) {
            $(".menu-container", "#" + network.name).toggle(false);
            var menu = $(evt.currentTarget).parent();
            if ($(evt.currentTarget).attr("nwAct") === "start") {
                $(".network-state", $("#" + network.name)).switchClass("down", "nw-loading");
                $("[nwAct='start']", menu).addClass("ui-state-disabled");
                $("[nwAct='delete']", menu).addClass("ui-state-disabled");
                $(":first-child", $("[nwAct='delete']", menu)).attr("disabled", true);
                kimchi.toggleNetwork(network.name, true, function() {
                    $("[nwAct='start']", menu).addClass("hide-action-item");
                    $("[nwAct='start']", menu).removeClass("ui-state-disabled");
                    $("[nwAct='stop']", menu).removeClass("hide-action-item");
                    if (network.in_use) {
                        $("[nwAct='stop']", menu).addClass("ui-state-disabled");
                    }
                    $(".network-state", $("#" + network.name)).switchClass("nw-loading", "up");
                }, function(err) {
                    $(".network-state", $("#" + network.name)).switchClass("nw-loading","down");
                    $("[nwAct='start']", menu).removeClass("ui-state-disabled");
                    if (!network.in_use) {
                        $("[nwAct='delete']", menu).removeClass("ui-state-disabled");
                    }
                    $(":first-child", $("[nwAct='delete']", menu)).removeAttr("disabled");
                    kimchi.message.error(err.responseJSON.reason);
                });
            } else if ($(evt.currentTarget).attr("nwAct") === "stop") {
                $(".network-state", $("#" + network.name)).switchClass("up", "nw-loading");
                $("[nwAct='stop']", menu).addClass("ui-state-disabled");
                kimchi.toggleNetwork(network.name, false, function() {
                    $("[nwAct='start']", menu).removeClass("hide-action-item");
                    $("[nwAct='stop']", menu).addClass("hide-action-item");
                    $("[nwAct='stop']", menu).removeClass("ui-state-disabled");
                    if (!network.in_use) {
                        $("[nwAct='delete']", menu).removeClass("ui-state-disabled");
                        $(":first-child", $("[nwAct='delete']", menu)).removeAttr("disabled");
                    }
                    $(".network-state", $("#" + network.name)).switchClass("nw-loading", "down");
                }, function(err) {
                    $(".network-state", $("#" + network.name)).switchClass("nw-loading", "up");
                    if (!network.in_use) {
                        $("[nwAct='stop']", menu).removeClass("ui-state-disabled");
                    }
                    kimchi.message.error(err.responseJSON.reason);
                });
            } else if ($(evt.currentTarget).attr("nwAct") === "delete") {
                kimchi.confirm({
                    title : i18n['KCHAPI6006M'],
                    content : i18n['KCHNET6002M'],
                    confirm : i18n['KCHAPI6002M'],
                    cancel : i18n['KCHAPI6003M']
                }, function() {
                    kimchi.deleteNetwork(network.name, function() {
                        $(evt.currentTarget).parents(".item").remove();
                    });
                }, null);
            }
        }
    });
    $(".column-action", "#" + network.name).children(":first").button({
        icons : {
            secondary : "action-button-icon"
        }
    }).click(function() {
        $(".menu-container", "#" + network.name).toggle();
        window.scrollBy(0, 150);
    });
    $(".menu-container", "#" + network.name).mouseleave(function() {
        $(".menu-container", "#" + network.name).toggle(false);
    });
};

kimchi.initNetworkCreation = function() {
    $("#networkAdd").on("click", function() {
        kimchi.openNetworkDialog(function() {
            var network = kimchi.getNetworkDialogValues();
            var data = {
                name : network.name,
                connection: network.type
            };
            if (network.type === kimchi.NETWORK_TYPE_BRIDGE) {
                data.connection = "bridge";
                data.interface = network.interface;
                if ($("#enableVlan").prop("checked")) {
                    data.vlan_id = network.vlan_id;
                    if (!(data.vlan_id >=1 && data.vlan_id <= 4094)) {
                        kimchi.message.error.code('KCHNET6001E');
                        return;
                    }
                }
            }
            kimchi.createNetwork(data, function(result) {
                network.state = result.state === "active" ? "up" : "down";
                network.interface = result.interface ? result.interface : i18n["KCHNET6001M"];
                network.addrSpace = result.subnet ? result.subnet : i18n["KCHNET6001M"];
                kimchi.addNetworkItem(network);
                $("#networkConfig").dialog("close");
            });
        });
    });
};

kimchi.initNetworkDialog = function() {

    buttonsObj= {};
    buttonsObj['id'] = "networkFormOk";
    buttonsObj['text'] = i18n.KCHAPI6005M;
    buttonsObj['class'] = "ui-button-primary";
    buttonsObj['disabled'] = true;
    buttonsObj['click'] = function() { };
    $("#networkConfig").dialog({
        autoOpen : false,
        modal : true,
        width : 600,
        draggable : false,
        resizable : false,
        closeText: "X",
        dialogClass : "network-ui-dialog remove-when-logged-off",
        open : function(){
            $(".ui-dialog-titlebar-close", $("#networkConfig").parent()).removeAttr("title");
        },
        beforeClose : function() {
            kimchi.cleanNetworkDialog();
        },
        buttons : [buttonsObj]
    });
    kimchi.setupNetworkFormEvent();
};

kimchi.openNetworkDialog = function(okCallback) {
    kimchi.getInterfaces(function(result) {
        var options = "";
        var nics = {};
        for (var i = 0; i < result.length; i++) {
            options += "<option value=" + result[i].name + ">" + result[i].name + "</option>";
            nics[result[i].name] = result[i];
        }
        $("#networkInterface").append(options);
        onChange = function() {
            if (nics[$("#networkInterface").val()].type === "bridge") {
                $("#enableVlan").prop("checked", false);
                $("#enableVlan").prop("disabled", true);
                $("#networkVlanID").val("");
                $("#networkVlanID").prop("disabled", true);
            } else {
                $("#enableVlan").prop("disabled", false);
            }
        };
        $("#networkInterface").on("change", onChange);
        onChange();
        kimchi.setDefaultNetworkType(result.length!==0);
    });
    $("#networkConfig").dialog({
        title : i18n.KCHNET6003M
    });
    $("#networkFormOk").on("click", function() {
        okCallback();
    });
    $("#enableVlan").on("click", function() {
        $("#networkVlanID").prop("disabled", !this.checked);
        if (!this.checked) {
            $("#networkVlanID").val("");
        }
    });
    $("#networkConfig").dialog("open");
};

kimchi.enableBridgeOptions = function(enable) {
    if (!enable) {
        $("#enableVlan").prop("checked", false);
        $("#networkVlanID").prop("disabled", true);
        $("#networkVlanID").val("");
        $("#networkInterface").val("");
        $("#bridge-options").slideUp(100);
    } else if (!$("#networkInterface").val()){
        $("#networkInterface").prop("selectedIndex", 0);
        $("#bridge-options").slideDown(100);
    }
};

kimchi.setDefaultNetworkType = function(isInterfaceAvail) {
    $("#networkTypeBri").prop("checked", isInterfaceAvail);
    $("#networkTypeBri").prop("disabled", !isInterfaceAvail);
    $("#networkTypeNat").prop("checked", !isInterfaceAvail);
    if (!isInterfaceAvail) {
        kimchi.enableBridgeOptions(false);
    } else {
        $("#bridge-options").slideDown(100);
    }
};

kimchi.getNetworkDialogValues = function() {
    var network = {
        name : $("#networkName").val(),
        type : $("input:radio[name=networkType]:checked").val()
    };
    if (network.type === kimchi.NETWORK_TYPE_BRIDGE) {
        network.interface = $("#networkInterface").val();
        network.vlan_id = parseInt($("#networkVlanID").val());
    }
    return network;
};

kimchi.cleanNetworkDialog = function() {
    $("#networkInterface").empty();
    $("input:text", "#networkConfig").val(null).removeClass("invalid-field");
    $("#networkTypeIso").prop("checked", false);
    $("#networkTypeNat").prop("checked", false);
    $("#networkTypeBri").prop("checked", false);
    $("#networkInterface").prop("disabled", false);
    $("#networkInterface option").removeAttr("selected").find(":first").attr("selected", "selected");
    $("#networkFormOk").off("click");
    $("#networkFormOk").button("disable");
    $("#networkVlanID").prop("disabled", true);
    $("#enableVlan").prop("checked", false);
};

kimchi.setupNetworkFormEvent = function() {
    $("#networkName").on("keyup", function(event) {
        $("#networkName").toggleClass("invalid-field", !$("#networkName").val().match(/^[a-zA-Z0-9_]+$/));
        kimchi.updateNetworkFormButton();
    });
    $("#networkTypeIso").on("click", function(event) {
        $("#networkInterface").prop("disabled", true);
        kimchi.enableBridgeOptions(false);
    });
    $("#networkTypeNat").on("click", function(event) {
        $("#networkInterface").prop("disabled", true);
        kimchi.enableBridgeOptions(false);
    });
    $("#networkTypeBri").on("click", function(event) {
        $("#networkInterface").prop("disabled", false);
        kimchi.enableBridgeOptions(true);
    });
};

kimchi.updateNetworkFormButton = function() {
    if($("#networkName").hasClass("invalid-field")){
        $("#networkFormOk").button("disable");
    }else{
        $("#networkFormOk").button("enable");
    }
};

kimchi.initNetworkCleanup = function() {
    $("#network-content").on("remove", function() {
        $("#networkConfig").dialog("destroy");
    });
};
/*
 * Project Kimchi
 *
 * Copyright IBM, Corp. 2013
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
kimchi.popable = function() {
    $(document).click(function(e) {
        $('.popable').removeClass('open');
    });
    $(document).on("click", ".popable", function(e) {
        var isOpen = $(this).hasClass('open');
        $(".popable").removeClass('open');
        if (!isOpen && $(this).find('ul').html() !== '') {
            $(this).addClass('open');
        }

        // Scroll the popup menu into viewport if invisible.
        var popMenu = $('.popover', this).get(0);
        false === kimchi.isElementInViewport(popMenu) &&
            popMenu.scrollIntoView();

        e.preventDefault();
        e.stopPropagation();
    });
};
kimchi.report_add_main = function() {
    var addReportForm = $('#form-report-add');
    var submitButton = $('#button-report-add');
    var nameTextbox = $('input[name="name"]', addReportForm);
    var submitForm = function(event) {
        if(submitButton.prop('disabled')) {
            return false;
        }
        var reportName = nameTextbox.val();
        var validator = RegExp("^[A-Za-z0-9-]*$");
        if (!validator.test(reportName)) {
            kimchi.message.error.code('KCHDR6011M');
            return false;
        }
        var formData = addReportForm.serializeObject();
        submitButton
            .text(i18n['KCHDR6007M'])
            .prop('disabled', true);
        nameTextbox.prop('disabled', true);
        kimchi.createReport(formData, function(result) {
            kimchi.window.close();
            kimchi.topic('kimchi/debugReportAdded').publish({
                result: result
            });
        }, function(result) {
            // Error message from Async Task status
            if (result['message']) {
                var errText = result['message'];
            }
            // Error message from standard kimchi exception
            else {
                var errText = result['responseJSON']['reason'];
            }
            result && kimchi.message.error(errText)
            submitButton
                .text(i18n['KCHDR6006M'])
                .prop('disabled', false);
            nameTextbox.prop('disabled', false).focus();
        });

        event.preventDefault();
    };

    addReportForm.on('submit', submitForm);
    submitButton.on('click', submitForm);
};
/*
 * Project Kimchi
 *
 * Copyright IBM, Corp. 2013
 *
 * Authors:
 *  Hongliang Wang <hlwang@linux.vnet.ibm.com>
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
kimchi.repository_add_main = function() {

    var addForm = $('#form-repository-add');
    var addButton = $('#button-repository-add');

    var validateField = function(event) {
        var valid=($(this).val()!=='');
        $(addButton).prop('disabled', !valid);
        return(valid);
    };

    var validateForm = function(event) {
        var valid=false;
        addForm.find('input.required').each( function() {
            valid=($(this).val()!=='');
            return(!valid);
        });
        return(valid);
    }

    addForm.find('input.required').on('input propertychange', validateField);

    var weedObject = function(obj) {
        for (var key in obj) {
            if (obj.hasOwnProperty(key)) {
                if((typeof(obj[key])==="object") && !Array.isArray(obj[key])) {
                    weedObject(obj[key]);
                }
                else if(obj[key] == '') {
                    delete obj[key];
                }
            }
        }
    }

    var addRepository = function(event) {
        var valid = validateForm();
        if(!valid) {
            return false;
        }

        var formData = $(addForm).serializeObject();

        if (formData && formData.isMirror!=undefined) {
            formData.isMirror=(String(formData.isMirror).toLowerCase() === 'true');
        }
        if(formData.isMirror) {
            if(formData.config==undefined) {
                formData.config=new Object();
            }
            formData.config.mirrorlist=formData.baseurl;
            delete formData.baseurl;
            delete formData.isMirror;
        }
        weedObject(formData);
        if(formData.config && formData.config.comps) {
            formData.config.comps=formData.config.comps.split(/[,\s]/);
            for(var i=0; i>formData.config.comps.length; i++) {
                formData.config.comps[i]=formData.config.comps[i].trim();
            }
            for (var j=formData.config.comps.indexOf(""); j!=-1; j=formData.config.comps.indexOf("")) {
                formData.config.comps.splice(j, 1);
            }
        }

        kimchi.createRepository(formData, function() {
            kimchi.topic('kimchi/repositoryAdded').publish();
            kimchi.window.close();
        }, function(jqXHR, textStatus, errorThrown) {
            var reason = jqXHR &&
                jqXHR['responseJSON'] &&
                jqXHR['responseJSON']['reason'];
            reason = reason ? ': ' + reason : '';
            kimchi.message.error(i18n['KCHREPO6015M'] + reason);
        });
        return false;
    };

    $(addForm).on('submit', addRepository);
};
/*
 * Project Kimchi
 *
 * Copyright IBM, Corp. 2013
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

kimchi.repository_fillForm = function(form, name, values) {
    var name= (name=="" || !name) ? "%s" : name;
    for(var prop in values) {
        if ((typeof(values[prop])==="object") && !Array.isArray(values[prop])) {
            tmpName=name.replace("%s", prop+"[%s]" );
            kimchi.repository_fillForm(form, tmpName, values[prop]);
        }
        else {
            tmpName=name.replace("%s", prop );
            var control = $('input[name="' + tmpName + '"]', form);
            switch($(control).attr('type')) {
            case 'text':
                $(control).val(values[prop]);
                break;
            case 'radio':
            case 'checkbox':
                $(control).each(function(i, c) {
                    var matched = ('' + values[prop]) == $(c).val();
                    $(c).prop('checked', matched);
                });
                break;
            default:
                break;
            }
        }
    }
}

kimchi.repository_edit_main = function() {

    var editForm = $('#form-repository-edit');

    var saveButton = $('#repository-edit-button-save');

    if(kimchi.host.capabilities['repo_mngt_tool']=="yum") {
        editForm.find('input.deb').prop('disabled', true);
    }
    else if(kimchi.host.capabilities['repo_mngt_tool']=="deb") {
        editForm.find('input.yum').prop('disabled', true);
    }

    kimchi.retrieveRepository(kimchi.selectedRepository, function(repository) {
        kimchi.repository_fillForm(editForm,"",repository);

        $('input', editForm).on('input propertychange', function(event) {
            if($(this).val() !== '') {
                $(saveButton).prop('disabled', false);
            }
        });
    });


    var editRepository = function(event) {
        var formData = $(editForm).serializeObject();

        if (formData && formData.config) {
            formData.config.gpgcheck=(String(formData.config.gpgcheck).toLowerCase() === 'true');
        }

        if(formData.config && formData.config.comps) {
            formData.config.comps=formData.config.comps.split(/[,\s]/);
            for(var i=0; i>formData.config.comps.length; i++) {
                formData.config.comps[i]=formData.config.comps[i].trim();
            }
            for (var j=formData.config.comps.indexOf(""); j!=-1; j=formData.config.comps.indexOf("")) {
                formData.config.comps.splice(j, 1);
            }
        }

        kimchi.updateRepository(kimchi.selectedRepository, formData, function() {
            kimchi.topic('kimchi/repositoryUpdated').publish();
            kimchi.window.close();
        }, function(jqXHR, textStatus, errorThrown) {
            var reason = jqXHR &&
                jqXHR['responseJSON'] &&
                jqXHR['responseJSON']['reason'];
            reason = reason ? ': ' + reason : '';
            kimchi.message.error(i18n['KCHREPO6015M'] + reason);
        });

        return false;
    };

    $(editForm).on('submit', editRepository);
    $(saveButton).on('click', editRepository);
};
/*
 * Project Kimchi
 *
 * Copyright IBM, Corp. 2013
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

kimchi.select = function(id, options) {
    var listControl = $('#'+ id);
    var targetId = listControl.data('target');
    var labelId = listControl.data('label');
    var value = $('#' + targetId).val();
    var item;
    var itemTag = 'li';
    var selectedClass = 'active';
    $.each(options, function(index, option) {
        item = $('<' + itemTag + '></' + itemTag + '>');
        item.text(option.label);
        item.data('value', option.value);
        if(option.value === value) {
            item.addClass(selectedClass);
            $('#' + labelId).text(option.label);
        }
        listControl.append(item);
    });

    listControl.on('click', itemTag, function() {
        listControl.children().removeClass(selectedClass);
        $(this).addClass(selectedClass);
        $('#' + labelId).text($(this).text());
        var target = $('#' + targetId);
        var oldValue = target.val();
        var newValue = $(this).data('value');
        target.val(newValue);
        if(oldValue !== newValue) {
            target.change();
        }
    });
};
/*
 * Project Kimchi
 *
 * Copyright IBM, Corp. 2013
 *
 * Licensed under the Apache License, Version 2.0 (the 'License');
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an 'AS IS' BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
kimchi.doListStoragePools = function() {
    kimchi.listStoragePools(function(result) {
        var storageHtml = $('#storageTmpl').html();
        if (result && result.length) {
            var listHtml = '';
            $.each(result, function(index, value) {
                value.usage = parseInt(value.allocated / value.capacity * 100) || 0;
                value.capacity = kimchi.changetoProperUnit(value.capacity,1);
                value.allocated = kimchi.changetoProperUnit(value.allocated,1);
                if ('kimchi-iso' !== value.type) {
                    listHtml += kimchi.template(storageHtml, value);
                }
            });
            $('#storagepoolsList').html(listHtml);
            kimchi.storageBindClick();
        } else {
            $('#storagepoolsList').html('');
        }
    }, function(err) {
        kimchi.message.error(err.responseJSON.reason);
    });

}

kimchi.storageBindClick = function() {

    $('.inactive').each(function(index) {
        if ('active' === $(this).data('state')) {
            $(this).hide();
        } else {
            $(this).show();
        }
    });

    $('.list-storage .storage-state .active').each(function(index) {
        if ('active' === $(this).data('state')) {
            $(this).show();
        } else {
            $(this).hide();
        }
    });

    $('.pool-activate').each(function(index) {
        if ('active' === $(this).data('stat')) {
            $(this).hide();
        } else {
            $(this).show();
        }
    });

    $('.pool-deactivate').each(function(index) {
        if ('active' === $(this).data('stat')) {
            $(this).show();
        } else {
            $(this).hide();
        }
    });

    $('.pool-delete').on('click', function(event) {
        var $pool = $(this);
        var settings = {
            title : i18n['KCHAPI6001M'],
            content : i18n['KCHPOOL6001M'],
            confirm : i18n['KCHAPI6002M'],
            cancel : i18n['KCHAPI6003M']
        };
        kimchi.confirm(settings, function() {
            var poolName = $pool.data('name');
            kimchi.deleteStoragePool(poolName, function() {
                kimchi.doListStoragePools();
            }, function(err) {
                kimchi.message.error(err.responseJSON.reason);
            });
        });
    });

    $('.pool-activate').on('click', function(event) {
        var poolName = $(this).data('name');
        kimchi.changePoolState(poolName, 'activate', function() {
            kimchi.doListStoragePools();
        }, function(err) {
            kimchi.message.error(err.responseJSON.reason);
        });
    });

    $('.pool-deactivate').on('click', function(event) {
        var poolName = $(this).data('name');
        var settings = {
            title : i18n['KCHAPI6001M'],
            content : i18n['KCHPOOL6012M'],
            confirm : i18n['KCHAPI6002M'],
            cancel : i18n['KCHAPI6003M']
        };
        if (!$(this).data('persistent')) {
            kimchi.confirm(settings, function() {
                kimchi.changePoolState(poolName, 'deactivate', function() {
                    kimchi.doListStoragePools();
                }, function(err) {
                    kimchi.message.error(err.responseJSON.reason);
                });
            }, function() {
                return false;
            });
        }
        else {
            kimchi.changePoolState(poolName, 'deactivate', function() {
                kimchi.doListStoragePools();
            }, function(err) {
                kimchi.message.error(err.responseJSON.reason);
            });
        }
    });

    $('.storage-action').on('click', function() {
        var storage_action = $(this);
        var deleteButton = storage_action.find('.pool-delete');
        if ('active' === deleteButton.data('stat')) {
            deleteButton.attr('disabled', 'disabled');
        } else {
            deleteButton.removeAttr('disabled');
        }
    });

    $('#volume-doAdd').on('click', function() {
        kimchi.window.open('storagevolume-add.html');
    });
    $('.storage-li').on('click', function(event) {
        if (!$(event.target).parents().hasClass('bottom')) {
            if ($(this).data('stat') === 'active') {
                var that = $(this);
                var volumeDiv = $('#volume' + that.data('name'));
                var slide = $(this).next('.volumes');
                if (that.hasClass('in')) {
                    kimchi.doListVolumes(that);
                } else {
                    slide.slideUp('slow');
                    that.addClass('in');
                    kimchi.changeArrow(that.children().last().children());
                }
            }
        }
    });
}

kimchi.doListVolumes = function(poolObj) {
    var volumeDiv = $('#volume' + poolObj.data('name'));
    var slide = poolObj.next('.volumes');
    var handleArrow = poolObj.children().last().children();
    kimchi.listStorageVolumes(poolObj.data('name'), function(result) {
        var volumeHtml = $('#volumeTmpl').html();
        if (result) {
            if (result.length) {
                var listHtml = '';
                $.each(result, function(index, value) {
                    value.poolname = poolObj.data('name');
                    value.capacity = kimchi.changetoProperUnit(value.capacity,1);
                    value.allocation = kimchi.changetoProperUnit(value.allocation,1);
                    listHtml += kimchi.template(volumeHtml, value);
                });
                volumeDiv.html(listHtml);
            } else {
                volumeDiv.html("<div class='pool-empty'>" + i18n['KCHPOOL6002M'] + "</div>");
            }
            poolObj.removeClass('in');
            kimchi.changeArrow(handleArrow);
            slide.slideDown('slow');
        }
    }, function(err) {
        kimchi.message.error(err.responseJSON.reason);
    });
}

kimchi.storage_main = function() {
    $('#storage-pool-add').on('click', function() {
        kimchi.window.open('storagepool-add.html');
    });
    kimchi.doListStoragePools();
}

kimchi.changeArrow = function(obj) {
    if ($(obj).hasClass('arrow-down')) {
        $(obj).removeClass('arrow-down').addClass('arrow-up');
    } else {
        $(obj).removeClass('arrow-up').addClass('arrow-down');
    }
}
/*
 * Project Kimchi
 *
 * Copyright IBM, Corp. 2013
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

kimchi.storagepool_add_main = function() {
    kimchi.initStorageAddPage();
    $('#form-pool-add').on('submit', kimchi.addPool);
    $('#pool-doAdd').on('click', kimchi.addPool);
};

kimchi.initStorageAddPage = function() {
    kimchi.listHostPartitions(function(data) {
        if (data.length > 0) {
            var deviceHtml = $('#partitionTmpl').html();
            var listHtml = '';
            valid_types = ['part', 'disk', 'mpath'];
            $.each(data, function(index, value) {
                if (valid_types.indexOf(value.type) != -1) {
                    listHtml += kimchi.template(deviceHtml, value);
                }
            });
            $('.host-partition').html(listHtml);
        } else {
            $('.host-partition').html(i18n['KCHPOOL6011M']);
            $('.host-partition').addClass('text-help');
        }
    });

    kimchi.getHostPCIDevices(function(data){
        if(data.length>0){
            for(var i=0;i<data.length;i++){
                data[i].label = data[i].name;
                data[i].value = data[i].name;
            }
            $('#scsiAdapter').selectMenu();
            $("input", "#scsiAdapter").val(data[0].name);
            $('#scsiAdapter').selectMenu("setData", data);
        } else {
            $('#scsiAdapter').html(i18n['KCHPOOL6005M']);
            $('#scsiAdapter').addClass('text-help');
        }
    });

    $('#poolTypeId').selectMenu();
    $('#serverComboboxId').combobox();
    $('#targetFilterSelectId').filterselect();
    var options = [ {
        label : "DIR",
        value : "dir"
    }, {
        label : "NFS",
        value : "netfs"
    }, {
        label : "iSCSI",
        value : "iscsi"
    }, {
        label : "LOGICAL",
        value : "logical"
    }, {
        label : i18n.KCHPOOL6004M,
        value : "scsi"
    } ];
    $('#poolTypeId').selectMenu("setData", options);
 
    kimchi.getStorageServers('netfs', function(data) {
        var serverContent = [];
        if (data.length > 0) {
            $.each(data, function(index, value) {
                serverContent.push({
                    label : value.host,
                    value : value.host
                });
            });
        }
        $('#serverComboboxId').combobox("setData", serverContent);
        $('input[name=nfsServerType]').change(function() {
            if ($(this).val() === 'input') {
                $('#nfsServerInputDiv').removeClass('tmpl-html');
                $('#nfsServerChooseDiv').addClass('tmpl-html');
            } else {
                $('#nfsServerInputDiv').addClass('tmpl-html');
                $('#nfsServerChooseDiv').removeClass('tmpl-html');
            }
        });
        $('#nfsserverId').on("change keyup",function() {
            if ($(this).val() !== '' && kimchi.isServer($(this).val())) {
                $('#nfspathId').prop('disabled',false);
                $(this).removeClass("invalid-field");
            } else {
                $(this).addClass("invalid-field");
                $('#nfspathId').prop( "disabled",true);
            }
            $('#targetFilterSelectId').filterselect('clear');
        });
        $('#nfspathId').focus(function() {
            var targetContent = [];
            kimchi.getStorageTargets($('#nfsserverId').val(), 'netfs', function(data) {
                if (data.length > 0) {
                    $.each(data, function(index, value) {
                        targetContent.push({
                            label : value.target,
                            value : value.target
                        });
                    });
                }
                $('#targetFilterSelectId').filterselect("setData", targetContent);
            });
        });
    });

    $('#poolTypeInputId').change(function() {
        if ($(this).val() === 'dir') {
            $('.path-section').removeClass('tmpl-html');
            $('.logical-section').addClass('tmpl-html');
            $('.nfs-section').addClass('tmpl-html');
            $('.iscsi-section').addClass('tmpl-html');
            $('.scsi-section').addClass('tmpl-html');
        } else if ($(this).val() === 'netfs') {
            $('.path-section').addClass('tmpl-html');
            $('.logical-section').addClass('tmpl-html');
            $('.nfs-section').removeClass('tmpl-html');
            $('.iscsi-section').addClass('tmpl-html');
            $('.scsi-section').addClass('tmpl-html');
        } else if ($(this).val() === 'iscsi') {
            $('.path-section').addClass('tmpl-html');
            $('.logical-section').addClass('tmpl-html');
            $('.nfs-section').addClass('tmpl-html');
            $('.iscsi-section').removeClass('tmpl-html');
            $('.scsi-section').addClass('tmpl-html');
        } else if ($(this).val() === 'logical') {
            $('.path-section').addClass('tmpl-html');
            $('.logical-section').removeClass('tmpl-html');
            $('.nfs-section').addClass('tmpl-html');
            $('.iscsi-section').addClass('tmpl-html');
            $('.scsi-section').addClass('tmpl-html');
        } else if ($(this).val() === 'scsi') {
            $('.scsi-section').removeClass('tmpl-html');
            $('.path-section').addClass('tmpl-html');
            $('.logical-section').addClass('tmpl-html');
            $('.nfs-section').addClass('tmpl-html');
            $('.iscsi-section').addClass('tmpl-html');
        }
    });
    $('#authId').click(function() {
        if ($(this).prop("checked")) {
            $('.authenticationfield').removeClass('tmpl-html');
        } else {
            $('.authenticationfield').addClass('tmpl-html');
        }
    });
    $('#iscsiportId').keyup(function(event) {
        $(this).toggleClass("invalid-field",!/^[0-9]+$/.test($(this).val()));
    });
};

kimchi.validateForm = function() {
    var name = $('#poolId').val();
    var poolType = $("#poolTypeInputId").val();
    if ('' === name) {
        kimchi.message.error.code('KCHPOOL6001E');
        return false;
    }
    if (name.indexOf("/")!=-1) {
        kimchi.message.error.code('KCHPOOL6004E');
        return false;
    }
    if (poolType === "dir") {
        return kimchi.validateDirForm();
    } else if (poolType === "netfs") {
        return kimchi.validateNfsForm();
    } else if (poolType === "iscsi") {
        return kimchi.validateIscsiForm();
    } else if (poolType === "logical") {
        return kimchi.validateLogicalForm();
    } else {
        return true;
    }
};

kimchi.validateDirForm = function () {
    var path = $('#pathId').val();
    if ('' === path) {
        kimchi.message.error.code('KCHPOOL6002E');
        return false;
    }
    if (!/(^\/.*)$/.test(path)) {
        kimchi.message.error.code('KCHAPI6003E');
        return false;
    }
    return true;
};

kimchi.validateNfsForm = function () {
    var nfspath = $('#nfspathId').val();
    var nfsserver = $('#nfsserverId').val();
    if (!kimchi.validateServer(nfsserver)) {
        return false;
    }
    if ('' === nfspath) {
        kimchi.message.error.code('KCHPOOL6003E');
        return false;
    }
    if (!/((\/([0-9a-zA-Z-_\.]+)))$/.test(nfspath)) {
        kimchi.message.error.code('KCHPOOL6005E');
        return false;
    }
    return true;
};

kimchi.validateIscsiForm = function() {
    var iscsiServer = $('#iscsiserverId').val();
    var iscsiTarget = $('#iscsiTargetId').val();
    if (!kimchi.validateServer(iscsiServer)) {
        return false;
    }
    if ('' === iscsiTarget) {
        kimchi.message.error.code('KCHPOOL6007E');
        return false;
    }
    return true;
};

kimchi.validateServer = function(serverField) {
    if ('' === serverField) {
        kimchi.message.error.code('KCHPOOL6008E');
        return false;
    }
    if(!kimchi.isServer(serverField)) {
        kimchi.message.error.code('KCHPOOL6009E');
        return false;
    }
    return true;
};

kimchi.validateLogicalForm = function () {
    if ($("input[name=devices]:checked").length === 0) {
        kimchi.message.error.code('KCHPOOL6006E');
        return false;
    } else {
        return true;
    }
};

kimchi.addPool = function(event) {
    if (kimchi.validateForm()) {
        var formData = $('#form-pool-add').serializeObject();
        delete formData.authname;
        var poolType = $('#poolTypeId').selectMenu('value');
        if (poolType === 'dir') {
            formData.path = $('#pathId').val();
        } else if (poolType === 'logical') {
            var source = {};
            if (!$.isArray(formData.devices)) {
                var deviceObj = [];
                deviceObj[0] =  formData.devices;
                source.devices = deviceObj;
            } else {
                source.devices = formData.devices;
            }
            delete formData.devices;
            formData.source = source;
        } else if (poolType === 'netfs'){
            var source = {};
            source.path = $('#nfspathId').val();
            source.host = $('#nfsserverId').val();
            formData.source = source;
        } else if (poolType === 'iscsi') {
            var source = {};
            source.target = $('#iscsiTargetId').val();
            source.host = $('#iscsiserverId').val();
            $('#iscsiportId').val() !== '' ? source.port = parseInt($('#iscsiportId').val()): null;
            if ($('#authId').prop("checked")) {
                source.auth = {
                    "username" : $('#usernameId').val(),
                    "password" : $('#passwordId').val()
                };
            }
            formData.source = source;
        } else if (poolType === 'scsi'){
            formData.source = { adapter_name: $('#scsiAdapter').selectMenu('value') };
        }
        if (poolType === 'logical') {
            var settings = {
                title : i18n['KCHAPI6001M'],
                content : i18n['KCHPOOL6003M'],
                confirm : i18n['KCHAPI6002M'],
                cancel : i18n['KCHAPI6003M']
            };
            kimchi.confirm(settings, function() {
                kimchi.createStoragePool(formData, function() {
                    kimchi.doListStoragePools();
                    kimchi.window.close();
                }, function(err) {
                    kimchi.message.error(err.responseJSON.reason);
                });
            }, function() {
            });
        } else {
            kimchi.createStoragePool(formData, function() {
                kimchi.doListStoragePools();
                kimchi.window.close();
            }, function(err) {
                kimchi.message.error(err.responseJSON.reason);
            });
        }
    }
};
/*
 * Project Kimchi
 *
 * Copyright IBM, Corp. 2013
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
kimchi.template = function(templateStr, data, tag) {
    tag = tag || /\{([^\}]+)\}/g;

    var escapeHtml = function(html) {
        return String(html)
            .replace(/&/g, '&amp;')
            .replace(/</g, '&lt;')
            .replace(/>/g, '&gt;')
            .replace(/"/g, '&quot;')
            .replace(/'/g, '&#39;');
    };

    return templateStr.replace(tag, function(matchResult, express) {
        var propertyArray = express.split('!');
        var defaultValue = propertyArray[1] || '';
        propertyArray = propertyArray[0].split('.');
        var value = data, i = 0, l = propertyArray.length, property;
        for (; i < l; i++) {
            property = propertyArray[i];
            if (value) {
                value = value[property];
            } else {
                break;
            }
        }
        return escapeHtml((value || value === 0) ? value : defaultValue);
    });
};
/*
 * Project Kimchi
 *
 * Copyright IBM, Corp. 2013
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
kimchi.switchPage = function(fromPageId, toPageId, direction) {
    direction = direction || 'left';
    var toLeftBegin;
    var fromLeftEnd;
    if('left' === direction) {
        toLeftBegin = '100%';
        fromLeftEnd = '-100%';
    } else if('right' === direction) {
        toLeftBegin = '-100%';
        fromLeftEnd = '100%';
    }
    var formPage = $('#'+fromPageId);
    var toPage = $('#'+toPageId);
    toPage.css({
        left: toLeftBegin
    });
    formPage.animate({
        left: fromLeftEnd,
        opacity: 0.1
    }, 400);
    toPage.animate({
        left: '0',
        opacity: 1
    }, 400);
};

kimchi.template_add_main = function() {
    kimchi.deepScanHandler = null;
    // 1-1 local iso
    $('#iso-local').click(function() {
        kimchi.switchPage('iso-type-box', 'iso-local-box');
        initLocalIsoField();
        initIsoFileField();
        kimchi.listIsos(function(isos) {
            if (isos && isos.length) {
                showLocalIsoField(isos);
                $('#iso-more').show();
            } else {
                $('#iso-search').show();
            }
        }, function(err) {
            kimchi.message.error(err.responseJSON.reason);
        });
    });

    $('#iso-local-box-back').click(function() {
        if (kimchi.deepScanHandler) {
            kimchi.deepScanHandler.stop = true;
        }
        kimchi.switchPage('iso-local-box', 'iso-type-box', 'right');
    });

    $('#iso-search').click(function() {
        var settings = {
            content : i18n['KCHTMPL6002M']
        };
        kimchi.confirm(settings, function() {
            $('#iso-search').hide();
            $('#iso-search-loading').show();
            deepScan('#iso-search');
        });
    });

    $('#iso-more').click(function() {
        var settings = {
            content : i18n['KCHTMPL6002M']
        };
        kimchi.confirm(settings, function() {
            $('#iso-more').hide();
            $('#iso-more-loading').show();
            deepScan('#iso-more');
        });
    });

    $('#iso-search-loading').click(function() {
        $('#iso-search-loading').hide();
        $('#iso-search').show();
        if (kimchi.deepScanHandler) {
            kimchi.deepScanHandler.stop = true;
        }
    });

    $('#iso-more-loading').click(function() {
        $('#iso-more-loading').hide();
        $('#iso-more').show();
        if (kimchi.deepScanHandler) {
            kimchi.deepScanHandler.stop = true;
        }
    });

    var deepScan = function(button) {
        kimchi.deepScanHandler = kimchi.stepListDeepScanIsos(function(isos, isFinished) {
            if (isos && isos.length) {
                if(button === '#iso-search') {
                    $(button + '-loading').hide();
                    button = '#iso-more';
                    $(button + '-loading').show();
                }
                showLocalIsoField(isos);
            } else {
                if (isFinished) {
                    kimchi.message.warn(i18n['KCHTMPL6001W']);
                }
            }
            if (isFinished) {
                $(button + '-loading').hide();
                $(button).show();
            }
        }, function(err) {
            kimchi.message.error(err.responseJSON.reason);
            $(button + '-loading').hide();
            $(button).show();
        });
    };

    //1-1-1 local iso list
    var initLocalIsoField = function() {
        kimchi.isoInfo = {};
        $('#local-iso-field').hide();
        $('#select-all-local-iso').prop('checked', false);
        $('#btn-template-local-iso-create').attr('disabled', 'disabled');
        $('#iso-search').hide();
        $('#iso-more').hide();
        $('#iso-search-loading').hide();
        $('#iso-more-loading').hide();
        $('#list-local-iso').empty();
    };

    var showLocalIsoField = function(isos) {
        var html = '';
        var template = $('#tmpl-list-local-iso').html();
        $.each(isos, function(index, volume) {
            var isoId = volume.os_distro + '*' + volume.name + '*' + volume.os_version;
            if (!kimchi.isoInfo[isoId]) {
                volume.isoId = isoId;
                volume.capacity = kimchi.changetoProperUnit(volume.capacity, 1);
                kimchi.isoInfo[isoId] = volume;
                html += kimchi.template(template, volume);
            }
        });
        $('#list-local-iso').append(html);
        $('#local-iso-field').show();
    };

    $('#select-all-local-iso').click(function() {
        $('#list-local-iso [type="checkbox"]').prop('checked', $(this).prop('checked'));
        if ($(this).prop('checked')) {
            $('#btn-template-local-iso-create').removeAttr('disabled');
        } else {
            $('#btn-template-local-iso-create').attr('disabled', 'disabled');
        }
    });

    $('#list-local-iso').on('click', '[type="checkbox"]', function() {
        var checkedLength = $('#list-local-iso [type="checkbox"]:checked').length;
        if (checkedLength) {
            $('#btn-template-local-iso-create').removeAttr('disabled');
            var length = $('#list-local-iso [type="checkbox"]').length;
            $('#select-all-local-iso').prop('checked', length == checkedLength);
        } else {
            $('#select-all-local-iso').prop('checked', false);
            $('#btn-template-local-iso-create').attr('disabled', 'disabled');
        }
    });

    $('#btn-template-local-iso-create').click(function() {
        submitIso('form-local-iso');
    });

    //1-1-2 local iso file
    var initIsoFileField = function() {
        $('#iso-file-check').prop('checked', false);
        $('#iso-file-box').hide();
        $('#iso-file').val('');
        $('#btn-template-file-create').attr('disabled', 'disabled');
    };

    $('#iso-file-check').click(function() {
        if ($(this).prop('checked')) {
            $('#iso-file-box').slideDown();
        } else {
            $('#iso-file-box').slideUp();
        }
    });

    $('#iso-file').on('input propertychange', function() {
        if ($('#iso-file').val()) {
            $('#btn-template-file-create').removeAttr('disabled');
        } else {
            $('#btn-template-file-create').attr('disabled', 'disabled');
        }
    });

    $('#btn-template-file-create').click(function() {
        var isoFile = $('#iso-file').val();
        if (!kimchi.template_check_path(isoFile)) {
            kimchi.message.error.code('KCHAPI6003E');
            return;
        }
        var data = {
            "cdrom" : isoFile
        };
        addTemplate(data);
    });

    //1-2 remote iso
    $('#iso-remote').css('opacity', 0.3).css('cursor', 'not-allowed');
    kimchi.getCapabilities(function(result) {
        if (result.qemu_stream == true) {
            $('#iso-remote').css('opacity', 1).css('cursor', 'pointer');

            $('#iso-remote').click(function() {
                kimchi.switchPage('iso-type-box', 'iso-remote-box');
                initRemoteIsoField();
                initIsoUrlField();
                kimchi.listDistros(function(isos) {
                    showRemoteIsoField(isos);
                }, function() {
                });
            });
        }
    }, function() {
    });

    $('#iso-remote-box-back').click(function() {
        kimchi.switchPage('iso-remote-box', 'iso-type-box', 'right');
    });

    //1-2-1 remote iso list
    var initRemoteIsoField = function() {
        $('#load-remote-iso').show();
        $('#remote-iso-field').hide();
        $('#iso-url-field').hide();
        $('#select-all-remote-iso').prop('checked', false);
        $('#btn-template-remote-iso-create').attr('disabled', 'disabled');
    };

    var showRemoteIsoField = function(isos) {
        if (isos && isos.length) {
            kimchi.isoInfo = {};
            var html = '';
            var template = $('#tmpl-list-remote-iso').html();
            $.each(isos, function(index, volume) {
                var isoId = volume.os_distro + '*' + volume.name + '*' + volume.os_version;
                if (!kimchi.isoInfo[isoId]) {
                    volume.isoId = isoId;
                    kimchi.isoInfo[isoId] = volume;
                    html += kimchi.template(template, volume);
                }
            });
            $('#list-remote-iso').html(html);
            $('#load-remote-iso').hide()
            $('#remote-iso-field').show();
            $('#iso-url-field').show();
        } else {
            $('#load-remote-iso').hide()
            $('#iso-url-field').show();
            kimchi.message.warn(i18n['KCHTMPL6001W']);
        }
    };

    $('#select-all-remote-iso').click(function() {
        $('#list-remote-iso [type="checkbox"]').prop('checked', $(this).prop('checked'));
        if ($(this).prop('checked')) {
            $('#btn-template-remote-iso-create').removeAttr('disabled');
        } else {
            $('#btn-template-remote-iso-create').attr('disabled', 'disabled');
        }
    });

    $('#list-remote-iso').on('click', '[type="checkbox"]', function() {
        var checkedLength = $('#list-remote-iso [type="checkbox"]:checked').length;
        if (checkedLength) {
            $('#btn-template-remote-iso-create').removeAttr('disabled');
            var length = $('#list-remote-iso [type="checkbox"]').length;
            $('#select-all-remote-iso').prop('checked', length == checkedLength);
        } else {
            $('#select-all-remote-iso').prop('checked', false);
            $('#btn-template-remote-iso-create').attr('disabled', 'disabled');
        }
    });

    $('#btn-template-remote-iso-create').click(function() {
        submitIso('form-remote-iso');
    });

    //1-2-2 remote iso url
    var initIsoUrlField = function() {
        $('#iso-url-check').prop('checked', false);
        $('#iso-url-box').hide();
        $('#iso-url').val('');
        $('#btn-template-url-create').attr('disabled', 'disabled');
    }

    $('#iso-url-check').click(function() {
        if ($(this).prop('checked')) {
            $('#iso-url-box').slideDown();
        } else {
            $('#iso-url-box').slideUp();
        }
    });

    $('#iso-url').on('input propertychange', function() {
        if ($('#iso-url').val()) {
            $('#btn-template-url-create').removeAttr('disabled');
        } else {
            $('#btn-template-url-create').attr('disabled', 'disabled');
        }
    });

    $('#btn-template-url-create').click(function() {
        var isoUrl = $('#iso-url').val();
        if (!kimchi.template_check_url(isoUrl)) {
            kimchi.message.error.code('KCHAPI6004E');
            return;
        }
        var data = {
            "cdrom" : isoUrl
        };
        addTemplate(data);
    });

    //do create
    var addTemplate = function(data) {
        kimchi.createTemplate(data, function() {
            kimchi.doListTemplates();
            kimchi.window.close();
            kimchi.topic('templateCreated').publish();
        }, function(err) {
            kimchi.message.error(err.responseJSON.reason);
        });
    };

    var submitIso = function(formId) {
        var formData = $('#' + formId).serializeObject();
        if (formData.iso) {
            var length = 0;
            var successNum = 0;
            var addTemplate = function(isoInfo) {
                var data = {
                    "os_distro" : isoInfo.os_distro,
                    "os_version" : isoInfo.os_version,
                    "cdrom" : isoInfo.path
                };
                kimchi.createTemplate(data, function() {
                    successNum++;
                    $('input[value="' + isoInfo.isoId + '"]').prop('checked', false);
                    $('.check-all>input').prop('checked', false);
                    kimchi.doListTemplates();
                    kimchi.topic('templateCreated').publish(data);
                    if (successNum === length) {
                        kimchi.window.close();
                    }
                }, function(err) {
                    kimchi.message.error(err.responseJSON.reason);
                });
            };
            if (formData.iso instanceof Array) {
                length = formData.iso.length;
                $.each(formData.iso, function(index, value) {
                    addTemplate(kimchi.isoInfo[value]);
                });
            } else {
                length = 1;
                addTemplate(kimchi.isoInfo[formData.iso]);
            }
        }
    };
};

kimchi.template_check_url = function(url) {
    var protocols = "((https|http|ftp|ftps|tftp)?://)?",
        userinfo = "(([0-9a-z_!~*'().&=+$%-]+:)?[0-9a-z_!~*'().&=+$%-]+@)?",
        ip = "(\\d{1,3}\.){3}\\d{1,3}",
        domain = "([0-9a-z_!~*'()-]+\.)*([0-9a-z][0-9a-z-]{0,61})?[0-9a-z]\.[a-z]{2,6}",
        port = "(:\\d{1,5})?",
        address = "(/[\\w!~*'().;?:@&=+$,%#-]+)+",
        domaintype = [ protocols, userinfo, domain, port, address ],
        ipType = [ protocols, userinfo, ip, port, address ],
        validate = function(type) {
            return new RegExp('^' + type.join('') + '$');
        };
    if (url.constructor === String) {
        return validate(domaintype).test(url) || validate(ipType).test(url);
    }
    return false;
};

kimchi.template_check_path = function(filePath) {
    var reg = /((\/([0-9a-zA-Z-_ \.]+))+)$/;
    if (filePath.constructor === String) {
        return reg.test(filePath);
    }
    return false;
};
/*
 * Project Kimchi
 *
 * Copyright IBM, Corp. 2013
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
kimchi.template_edit_main = function() {
    var templateEditForm = $('#form-template-edit');
    var origDisks;
    $('#template-name', templateEditForm).val(kimchi.selectedTemplate);
    kimchi.retrieveTemplate(kimchi.selectedTemplate, function(template) {
        origDisks =  template.disks;
        for ( var prop in template) {
            var value = template[prop];
            if (prop == 'graphics') {
               value = value["type"];
            }
            $('input[name="' + prop + '"]', templateEditForm).val(value);
        }
        var disks = template.disks;
        $('input[name="disks"]').val(disks[0].size);

        var options = [{label: 'VNC', value: 'vnc'}];
        kimchi.getCapabilities(function(result) {
            if (result.qemu_spice == true) {
                options.push({label: 'Spice', value: 'spice'})
            }
        }, function() {
        }, function(){
            kimchi.select('template-edit-graphics-list', options);
        });

        var scsipools = {};
        kimchi.listStoragePools(function(result) {
            var options = [];
            if (result && result.length) {
                $.each(result, function(index, storagePool) {
                    if ((storagePool.state=="active") && (storagePool.type !== 'kimchi-iso')) {
                        if ((storagePool.type == 'iscsi') || (storagePool.type == 'scsi')){
                            scsipools[storagePool.name] = [];
                            kimchi.listStorageVolumes(storagePool.name, function(result) {
                                if (result && result.length) {
                                    $.each(result, function(index, storageVolume) {
                                        options.push({
                                            label: storagePool.name + '/' + storageVolume.name,
                                            value: '/storagepools/' + storagePool.name + '/' + storageVolume.name
                                        });
                                        scsipools[storagePool.name].push(storageVolume)
                                    });
                                }
                                kimchi.select('template-edit-storagePool-list', options);
                            });
                        }
                        else {
                            options.push({
                                label: storagePool.name,
                                value: '/storagepools/' + storagePool.name
                            });
                        }
                    }
                });
            }
            if ($.isEmptyObject(scsipools)) {
                kimchi.select('template-edit-storagePool-list', options);
            }
        });
        kimchi.listNetworks(function(result) {
            if(result && result.length > 0) {
                var html = '';
                var tmpl = $('#tmpl-network').html();
                $.each(result, function(index, network) {
                    html += kimchi.template(tmpl, network);
                });
                $('#template-edit-network-list').html(html).show();
                if(template.networks && template.networks.length > 0) {
                    $('input[name="networks"]', templateEditForm).each(function(index, element) {
                        var value = $(element).val();
                        if(template.networks.indexOf(value) >= 0) {
                            $(element).prop('checked', true);
                        }
                    });
                }
            } else {
                $('#template-edit-network-list').hide();
            }
        });
    });

    $('#tmpl-edit-button-cancel').on('click', function() {
        kimchi.window.close();
    });

    $('#tmpl-edit-button-save').on('click', function() {
        var editableFields = [ 'name', 'cpus', 'memory', 'storagepool', 'disks', 'graphics'];
        var data = {};
        $.each(editableFields, function(i, field) {
            /* Support only 1 disk at this moment */
            if (field == 'disks') {
               origDisks[0].size = Number($('#form-template-edit [name="' + field + '"]').val());
               data[field] = origDisks;
            }
            else if (field == 'graphics') {
               var type = $('#form-template-edit [name="' + field + '"]').val();
               data[field] = {'type': type};
            }
            else {
               data[field] = $('#form-template-edit [name="' + field + '"]').val();
            }
        });
        data['memory'] = Number(data['memory']);
        data['cpus']   = Number(data['cpus']);
        storagepool = data['storagepool'];
        storageArray = storagepool.split("/");
        if (storageArray.length > 3){
            /* Support only 1 disk at this moment */
            delete data["disks"][0].size;
            data["disks"][0].volume = storageArray.pop();
            data['storagepool'] = storageArray.join("/");
        }
        var networks = templateEditForm.serializeObject().networks;
        if (networks instanceof Array) {
            data.networks = networks;
        } else if (networks != null) {
            data.networks = [networks];
        } else {
            data.networks = [];
        }

        kimchi.updateTemplate($('#template-name').val(), data, function() {
            kimchi.doListTemplates();
            kimchi.window.close();
        }, function(err) {
            kimchi.message.error(err.responseJSON.reason);
        });
    });
};
/*
 * Project Kimchi
 *
 * Copyright IBM, Corp. 2013
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
kimchi.doListTemplates = function() {
    kimchi.listTemplates(function(result) {
        if (result && result.length) {
            $('#noTemplates').hide();
            var listHtml = '';
            var templateHtml = $('#templateTmpl').html();
            $.each(result, function(index, value) {
                listHtml += kimchi.template(templateHtml, value);
            });
            $('#templateList').html(listHtml);
            kimchi.bindClick();
        } else {
            $('#templateList').html('');
            $('#noTemplates').show();
        }
    }, function(err) {
        kimchi.message.error(err.responseJSON.reason);
    });
};

kimchi.bindClick = function() {
    $('.template-edit').on('click', function(event) {
        var templateName = $(this).data('template');
        kimchi.selectedTemplate = templateName;
        kimchi.window.open("template-edit.html");
    });
    $('.template-delete').on('click', function(event) {
        var $template = $(this);
        var settings = {
            title : i18n['KCHAPI6001M'],
            content : i18n['KCHTMPL6003M'],
            confirm : i18n['KCHAPI6002M'],
            cancel : i18n['KCHAPI6003M']
        };
        kimchi.confirm(settings, function() {
            var templateName = $template.data('template');
            kimchi.deleteTemplate(templateName, function() {
                kimchi.doListTemplates();
            }, function(err) {
                kimchi.message.error(err.responseJSON.reason);
            });
        }, function() {
        });
    });
}
kimchi.hideTitle = function() {
    $('#tempTitle').hide();
};

kimchi.template_main = function() {
    $("#template-add").on("click", function(event) {
        kimchi.window.open({
            url: 'template-add.html',
            close: function() {
                if (kimchi.deepScanHandler) {
                    kimchi.deepScanHandler.stop = true;
                }
            }
        });
    });
    kimchi.doListTemplates();
};
/*
 * Project Kimchi
 *
 * Copyright IBM, Corp. 2013
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/**
 * pub/sub
 * Usage:
 *   Publish - kimchi.topic('eventname').publish(params);
 *   Subscribe - kimchi.topic('eventname').subscribe(listener);
 *   Unsubscribe - kimchi.topic('eventname').unsubscribe(listener);
 */
kimchi.topic = (function() {

    var topics = {};

    return function( id ) {
        var callbacks,
            method,
            topic = id && topics[ id ];

        if ( !topic ) {
            callbacks = jQuery.Callbacks();
            topic = {
                publish: callbacks.fire,
                subscribe: callbacks.add,
                unsubscribe: callbacks.remove
            };
            if ( id ) {
                topics[ id ] = topic;
            }
        }
        return topic;
    };
})();
/*
 * Project Kimchi
 *
 * Copyright IBM, Corp. 2013
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
kimchi.user = (function() {
    var getUserID = function() {
        return kimchi.cookie.get('userid');
    };

    var setUserID = function(userID) {
        kimchi.cookie.set('userid', userID, 365);
    };

    var showUser = function(toShow) {
        if (toShow) {
            var userName = getUserID();
            userName && $('#user-name').text(userName);
            $('#user').removeClass('not-logged-in');
            return;
        }

        $('#user').addClass('not-logged-in');
    };

    return {
        getUserID: getUserID,
        setUserID: setUserID,
        showUser: showUser
    };
})();
/*
 * Project Kimchi
 *
 * Copyright IBM, Corp. 2013
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */

/**
 * To check whether a given DOM node is entirely within the viewport
 * of a browser.
 * @param {DOMObject} el The given DOM node to check.
 *
 * @returns {true|false|undefined}
 *     true if the node is within viewport, or
 *     false if the node is not entirely visible, or
 *     undefined if the given parameter is invalid.
 */
kimchi.isElementInViewport = function(el) {
    if (!el || !el.getBoundingClientRect) {
        return undefined;
    }

    var rect = el.getBoundingClientRect();

    return (
        rect.top >= 0 &&
        rect.left >= 0 &&
        rect.bottom <= (window.innerHeight || document.documentElement.clientHeight) &&
        rect.right <= (window.innerWidth || document.documentElement.clientWidth)
    );
};

/**
 * To change the byte to proper unit.
 * @param number needed to change unit.
 * @param digits after the decimal point.
 * @returns str with unit.
 */
kimchi.changetoProperUnit = function(numOrg, digits, base) {
    if (numOrg === undefined) {
        return "";
    }
    var suffixes = [ 'K', 'M', 'G', 'T', 'P', 'E', 'Z', 'Y', 'B' ];
    var base = base || 1024;
    var numTemp = numOrg;
    var result = numOrg.toFixed(digits) + 'B';
    for ( var i = 0; i < suffixes.length; i++) {
        var numTemp = numTemp / base;
        if (numTemp < 1)
            break;
        result = numTemp.toFixed(digits) + suffixes[i]
    }
    return result;
};

/**
 * kimchi.formatMeasurement util.
 *
 * Refer to "Units of information" (
 *   http://en.wikipedia.org/wiki/Units_of_information
 * ) for more information about measurement units.
 *
 * @param number The number to be normalized.
 * @param settings
 *     * base Measurement base, accepts 2 or 10. defaults to 2.
 *     * unit The unit of the measurement, e.g., B, Bytes/s, bps, etc.
 *     * fixed The number of digits after the decimal point.
 *
 * @returns [object]
 *     * v The number part of the measurement.
 *     * s The suffix part of the measurement including multiple and unit.
 *         e.g., kB/s means 1000B/s, KiB/s for 1024B/s.
 */
(function() {
    var unitBaseMapping = {
        2: [{
            us: 'Ki',
            v: 1024
        }, {
            us: 'Mi',
            v: 1048576
        }, {
            us: 'Gi',
            v: 1073741824
        }, {
            us: 'Ti',
            v: 1099511627776
        }, {
            us: 'Pi',
            v: 1125899906842624
        }],
        10: [{
            us: 'k',
            v: 1000
        }, {
            us: 'M',
            v: 1000000
        }, {
            us: 'G',
            v: 1000000000
        }, {
            us: 'T',
            v: 1000000000000
        }, {
            us: 'P',
            v: 1000000000000000
        }]
    };

    var Formatted = function(value, suffix) {
        this['v'] = value;
        this['s'] = suffix;
    };
    Formatted.prototype.toString = function() {
        return this['v'] + this['s'];
    };

    var format = function(number, settings) {
        if(number === (undefined || null) || isNaN(number)) {
            return number;
        }

        var settings = settings || {};
        var unit = settings['unit'] || 'B';
        var base = settings['base'] || 2;
        if(base !== 2 && base !== 10) {
            return new Formatted(number, unit);
        }

        var fixed = settings['fixed'];

        var unitMapping = unitBaseMapping[base];
        for(var i = unitMapping.length - 1; i >= 0; i--) {
            var mapping = unitMapping[i];
            var suffix = mapping['us'];
            var startingValue = mapping['v'];
            if(number < startingValue) {
                continue;
            }

            var formatted = number / startingValue;
            formatted = fixed ? formatted.toFixed(fixed) : formatted;

            return new Formatted(formatted, suffix + unit);
        }

        return new Formatted(fixed ? number.toFixed(fixed) : number, unit);
    };

    kimchi.formatMeasurement = format;
})();

kimchi.isUnsignedNumeric = function(number) {
    var reg = /^d+(.d+)?$/
    return reg.test(number);
}

kimchi.isServer = function(server) {
    var domain = "([0-9a-z_!~*'()-]+\.)*([0-9a-z][0-9a-z-]{0,61})?[0-9a-z]\.[a-z]{2,6}";
    var ip = "(\\d{1,3}\.){3}\\d{1,3}";
    regex = new RegExp('^' + domain + '|' + ip + '$');
    if (!regex.test(server)) {
        return false;
    } else {
        return true;
    }
};
/*
 * Project Kimchi
 *
 * Copyright IBM, Corp. 2013
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
kimchi.window = (function() {
    var _windows = [];
    var _listeners = {};
    var open = function(settings) {
        var settings = jQuery.type(settings) === 'object' ? settings : {
            url: settings
        };

        var windowID = settings['id'] || 'window-' + _windows.length;

        if ($('#' + windowID).length) {
            $('#' + windowID).remove();
        }

        _windows.push(windowID);
        _listeners[windowID] = settings['close'];
        var windowNode = $('<div></div>', {
            id: windowID,
            'class': settings['class'] ? settings['class'] + ' bgmask remove-when-logged-off' : 'bgmask remove-when-logged-off'
        });

        $(windowNode).css(settings['style'] || '');

        $(windowNode).appendTo('body').on('click', '.window .close', function() {
            kimchi.window.close();
        });

        if (settings['url']) {
            $(windowNode).load(settings['url']).fadeIn(100);
            return;
        }

        settings['content'] && $(windowNode).html(settings['content']);
    };

    var close = function() {
        var windowID = _windows.pop();
        if(_listeners[windowID]) {
            _listeners[windowID]();
            _listeners[windowID] = null;
        }
        delete _listeners[windowID];

        $('#' + windowID).fadeOut(100, function() {
            $(this).remove();
        });
    };

    return {
        open: open,
        close: close
    };
})();
/*
 * Project Kimchi
 *
 * Copyright IBM, Corp. 2013
 *
 * Licensed under the Apache License, Version 2.0 (the 'License');
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an 'AS IS' BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
(function($) {
    $.widget('kimchi.dropdownButton', {

        _create : function() {
            this.actionDiv = this.element;
            this.actionDiv.addClass('btn dropdown popable');
            this.actionDiv.find('div').addClass('popover');
            this.actionDiv.find('button').addClass('button-big');
        },

        destroy : function() {
            this.actionDiv.removeClass('btn dropdown popable');
            this.actionDiv.find('div').removeClass('popover');
            this.actionDiv.find('button').removeClass('button-big');
            $.Widget.prototype.destroy.call(this);
        }
    });
}(jQuery));
/*
 * Project Kimchi
 *
 * Copyright IBM, Corp. 2013
 *
 * Licensed under the Apache License, Version 2.0 (the "License");
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an "AS IS" BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
 (function($) {
    $.widget('kimchi.circleGauge', {

        options : {
            color : '#87C004',
            fillColor : '#87C004',
            lineWidth : 20,
            shadowSize : '2px',
            font : 'bold 13px Geneva, sans-serif',
            textAlign : 'center',
            radius : 35,
            peakRate : 100,
            display : 0,
            circle : 0,
            label : ''
        },

        _create : function() {
            //valuesAttr="{" + this.element.data('value')+ "}";
            //console.info(valuesAttr);
            //values=eval("(" + valuesAttr + ")");
            //$.extend(this.options, values);
            this.options.display=this.element.data('display');
            this.options.percentage=this.element.data('percentage');
            this._fixupPeakRate();
            this._draw();
        },

        setValues : function(values) {
            $.extend(this.options, values);
            this._fixupPeakRate();
            this._draw();
        },

        _fixupPeakRate : function() {
            if (this.options.circle>this.options.peakRate) {
                this.options.peakRate=this.options.circle;
            }
        },

        _draw : function() {
            this.element.empty();
            var canvas = document.createElement('canvas');
            //this.element.append($(canvas));  //I don't quite understand this line so trying the one below...
            this.element.append(canvas);

            var ctx = canvas.getContext('2d');
            var radius = this.options.radius;

            var shadowSize = 2;
            var width = height = radius * 2;
            $(canvas).attr('height', height);
            $(canvas).attr('width', width);

            $(canvas).css({
                'boxShadow' : shadowSize + 'px ' + shadowSize + 'px ' + shadowSize + 'px #fff, -' + shadowSize + 'px -' + shadowSize + 'px ' + shadowSize + 'px #eaeaea',
                borderRadius : radius + 'px'
            });

            ctx.clearRect(0, 0, width, height);
            ctx.fillStyle = this.options.fillColor;
            ctx.font = this.options.font;
            ctx.textAlign = 'center';
            var originPos = radius;
            ctx.textBaseline = 'middle';
            ctx.fillText(this.options.display, originPos, originPos);
            ctx.strokeStyle = this.options.color;
            ctx.lineWidth = this.options.lineWidth;
            ctx.beginPath();
            ctx.arc(originPos, originPos, radius, -.5 * Math.PI, (this.options.percentage / 50 - .5) * Math.PI);
            ctx.stroke();
        },

        destroy : function() {
            this.element.empty();
            $.Widget.prototype.destroy.call(this);
        }
    });
}(jQuery));

kimchi.circleGauge = function(selector) {
    $(selector).circleGauge();
};
/*
 * Project Kimchi
 *
 * Copyright IBM, Corp. 2013
 *
 * Licensed under the Apache License, Version 2.0 (the 'License');
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an 'AS IS' BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
(function($) {
    $.widget('kimchi.combobox', {
        _create : function() {
            this.selectDiv = this.element;
            this.listControl = this.selectDiv.find('ul').first();
            this.listControl.html('');
            this.target = this.selectDiv.find('input').first();
            this.selectDiv.addClass('btn-select dropdown popable');
            this.target.addClass('input');
            this.target.width(this.selectDiv.width()+10);
            this.listControl.addClass('select-list');
            this.listControl.parent().addClass('popover');
        },

        setData : function(options) {
            var that = this;
            var value = this.target.val();
            var selectedClass = 'active';
            var itemTag = 'li';
            if (options.length > 0) {
                that.target.after($('<span class="arrow"></span>'));
                that.listControl.on('click', itemTag, function(e) {
                    that.listControl.children().removeClass(selectedClass);
                    $(this).addClass(selectedClass);
                    var oldValue = that.target.val();
                    var newValue = $(this).data('value');
                    that.target.val(newValue);
                    if (oldValue !== newValue) {
                        that.target.change();
                    }
                });

                that.selectDiv.click(function(e) {
                    that.listControl.html('');
                    var items = that._dataList(options);
                    $.each(items, function(index, item) {
                        that.listControl.append(item);
                    })
                });

                that.target.keyup(function(event) {
                    that.listControl.html('');
                    var items = that._dataList(options);
                    var temp = 0;
                    $.each(items, function(index, item) {
                        if (item.text().indexOf(that.target.val()) == 0) {
                            that.listControl.append(item);
                            temp++;
                        }
                    });
                    if (temp > 0 && that.listControl.html() !== '') {
                        that._open();
                    } else {
                        that._close();
                    }
                });
            }
        },

        value : function(value) {
            if (value === undefined) {
                return this.target.val();
            }
            this.target.val(value);
        },

        _dataList : function(options) {
            var item;
            var itemTag = 'li';
            var selectedClass = 'active';
            var items = [];
            var that = this;
            $.each(options, function(index, option) {
                item = $('<' + itemTag + '>' + option.label +'</' + itemTag + '>');
                item.data('value', option.value);
                if (option.value === that.target.val()) {
                    item.addClass(selectedClass);
                }
                items.push(item);
            });
            return items;
        },

        clear : function() {
            this.target.val("");
        },

        _open : function() {
            var isOpen = this.selectDiv.hasClass('open');
            if (!isOpen) {
                this.selectDiv.addClass('open');
            }
        },

        _close : function() {
            var isOpen = this.selectDiv.hasClass('open');
            if (isOpen) {
                this.selectDiv.removeClass('open');
            }
        },

        destroy : function() {
            this.selectDiv.removeClass('btn-select dropdown popable');
            this.target.removeClass('input');
            this.listControl.removeClass('select-list');
            this.listControl.parent().removeClass('popover');
            $.Widget.prototype.destroy.call(this);
        }
    });
}(jQuery));
/*
 * Project Kimchi
 *
 * Copyright IBM, Corp. 2013
 *
 * Licensed under the Apache License, Version 2.0 (the 'License');
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an 'AS IS' BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
(function($) {
    $.widget('kimchi.filterselect', {
        _create : function() {
            this.selectDiv = this.element;
            this.listControl = this.selectDiv.find('ul').first();
            this.listControl.html('');
            this.target = this.selectDiv.find('input').first();
            this.selectDiv.addClass('btn-select dropdown popable');
            this.target.addClass('input');
            this.target.width(this.selectDiv.width()+10);
            this.listControl.addClass('select-list');
            this.listControl.parent().addClass('popover');
        },

        setData : function(options) {
            var that = this;
            var value = this.target.val();
            var selectedClass = 'active';
            var itemTag = 'li';
            if (options.length > 0) {
                that.target.after($('<span class="arrow"></span>'));
                that.listControl.on('click', itemTag, function(e) {
                    that.listControl.children().removeClass(selectedClass);
                    $(this).addClass(selectedClass);
                    var oldValue = that.target.val();
                    var newValue = $(this).data('value');
                    that.target.val(newValue);
                    if (oldValue !== newValue) {
                        that.target.change();
                        that.target.removeClass("invalid-field");
                    }
                });

                that.selectDiv.click(function(e) {
                    that.listControl.html('');
                    var items = that._dataList(options);
                    if (items.length === 0) {
                        that.listControl.html(i18n['KCHAPI6005E']);
                    } else {
                        $.each(items, function(index, item) {
                            that.listControl.append(item);
                        });
                    }
                });

                that.target.keyup(function(event) {
                    that.listControl.html('');
                    var items = that._dataList(options);
                    var temp = 0;
                    $.each(items, function(index, item) {
                        if (item.text().indexOf(that.target.val()) == 0) {
                            that.listControl.append(item);
                            temp++;
                        }
                    });
                    if (that.listControl.html() === '') {
                        that.target.addClass("invalid-field");
                        that.listControl.html(i18n['KCHAPI6005E']);
                    } else {
                        that.target.removeClass("invalid-field");
                    }
                    if (temp > 0) {
                        that._open();
                    }
                });
            }
        },

        value : function(value) {
            if (value === undefined) {
                return this.target.val();
            }
            this.target.val(value);
        },

        _dataList : function(options) {
            var item;
            var itemTag = 'li';
            var selectedClass = 'active';
            var items = [];
            var that = this;
            $.each(options, function(index, option) {
                item = $('<' + itemTag + '>' + option.label +'</' + itemTag + '>');
                item.data('value', option.value);
                if (option.value === that.target.val()) {
                    item.addClass(selectedClass);
                }
                items.push(item);
            });
            return items;
        },

        clear : function() {
            this.target.val("");
        },

        _open : function() {
            var isOpen = this.selectDiv.hasClass('open');
            if (!isOpen) {
                this.selectDiv.addClass('open');
            }
        },

        _close : function() {
            var isOpen = this.selectDiv.hasClass('open');
            if (isOpen) {
                this.selectDiv.removeClass('open');
            }
        },

        destroy : function() {
            this.selectDiv.removeClass('btn-select dropdown popable');
            this.target.removeClass('input');
            this.listControl.removeClass('select-list');
            this.listControl.parent().removeClass('popover')
            $.Widget.prototype.destroy.call(this);
        }
    });
}(jQuery));
/*
 * Project Kimchi
 *
 * Copyright IBM, Corp. 2013
 *
 * Licensed under the Apache License, Version 2.0 (the 'License');
 * you may not use this file except in compliance with the License.
 * You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software
 * distributed under the License is distributed on an 'AS IS' BASIS,
 * WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
 * See the License for the specific language governing permissions and
 * limitations under the License.
 */
(function($) {
    $.widget('kimchi.selectMenu', {

        _create : function() {
            this.selectDiv = this.element;
            this.listControl = this.selectDiv.find('ul').first();
            this.listControl.html('');
            this.target = this.selectDiv.find('input').first();
            this.label = this.selectDiv.find('span').first();
            this.selectDiv.addClass('btn dropdown popable');
            this.target.addClass('input');
            this.label.addClass('input');
            this.listControl.addClass('select-list');
            this.listControl.parent().addClass('popover');
        },

        setData : function (options) {
            var that = this;
            var value = this.target.val();
            var selectedClass = 'active';
            var itemTag = 'li';
            var item;
            if (options.length > 0) {
                $.each(options, function(index, option) {
                    item = $('<' + itemTag + '>' + option.label +'</' + itemTag + '>');
                    item.data('value', option.value);
                    if(option.value === value) {
                        item.addClass(selectedClass);
                        that.label.text(option.label);
                    }
                    that.listControl.append(item);
                });
                that.listControl.on('click', itemTag, function() {
                    that.listControl.children().removeClass(selectedClass);
                    $(this).addClass(selectedClass);
                    that.label.text($(this).text());
                    var oldValue = that.target.val();
                    var newValue = $(this).data('value');
                    that.target.val(newValue);
                    if(oldValue !== newValue) {
                        that.target.change();
                    }
                });
            } else {
                kimchi.message.code.error('KCHAPI6006E');
            }
        },

        value : function(data) {
            if (data === undefined) {
                return this.target.val();
            }
            this.target.val(data.value);
            this.label.val(data.label);
        },

        destroy : function() {
            this.selectDiv.removeClass('btn dropdown popable');
            this.target.removeClass('input');
            this.label.removeClass('input');
            this.listControl.removeClass('select-list');
            this.listControl.parent().removeClass('popover');
            $.Widget.prototype.destroy.call(this);
        }
    });
}(jQuery));
